@@ -2,7 +2,7 @@
 //------------------------------------------------------------------------------
 //                              Card.h
 //------------------------------------------------------------------------------
-// $Id: Card.h,v 1.17 2008/01/14 03:41:00 vlg Exp $
+// $Id: Card.h,v 1.18 2008/05/23 03:01:38 vlg Exp $
 //
 // Date: Jan 8, 2004
 //------------------------------------------------------------------------------
@@ -36,8 +36,7 @@ public:
 	virtual ustring get_image_path   () const { return m_image_path;   }
 	virtual ustring get_answer       () const { return m_answer;       }
 	virtual ustring get_example      () const { return m_example;      }
-	virtual ustring get_example_image_path   () const 
-		{ return m_example_image_path;   }
+	virtual ustring get_example_image_path () const;
 	virtual ustring get_alt_spelling (SideSelection side_) const;
 	virtual long    get_id           () const { return m_id;           }
 
@@ -65,6 +64,12 @@ private:
 //------------------------------------------------------------------------------
 // Implementation
 //------------------------------------------------------------------------------
+inline ustring 
+Card::get_example_image_path () const 
+{ 
+	return m_example_image_path;
+}
+
 inline ustring
 Card::get_alt_spelling (SideSelection side_) const 
 { 
