@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: CardBox.cpp,v 1.77 2008/01/07 01:40:04 vlg Exp $
+// $Id: CardBox.cpp,v 1.81 2008/08/19 02:52:46 vlg Exp $
 //------------------------------------------------------------------------------
 //                            CardBox.cpp
 //------------------------------------------------------------------------------
@@ -19,7 +19,7 @@
 #include <limits.h>				// PATH_MAX
 #include <string.h>				// strcpy(3)
 
-#if !defined (WIN32)
+#if !defined (IS_WIN32)
 #  include <libgen.h>				// basename(3), dirname(3)
 #endif
 
@@ -125,7 +125,6 @@ CardBox (MainWindow& mwin_) : m_mwin (mwin_)
 {
 	trace_with_mask("CardBox::CardBox",GUITRACE);
 
-	Gtk::EventBox* evtbox = NULL;
 	Glib::ustring pixpath;
 	int i;
 
@@ -214,7 +213,7 @@ mask_out_unused_boxes ()
 	trace_with_mask("CardBox::mask_out_unused_boxes",GUITRACE);
 
 
-	int i = m_sched_db.capacity ();
+	u_int i = m_sched_db.capacity ();
 	for (; i < CARDBOXES_MAX && i < m_drawer_ui.size (); i++) 
 	{
 		m_drawer_ui [i].m_sched_vbox->hide ();
@@ -227,7 +226,7 @@ void
 CardBox::
 expose_hidden_box (int idx_)
 {
-	if (idx_ < m_drawer_ui.size ()) 
+	if ((u_int) idx_ < m_drawer_ui.size ()) 
 	{
 		m_drawer_ui [idx_].m_sched_vbox->show ();
 		m_drawer_ui [idx_].m_evtbox->show ();
@@ -371,9 +370,9 @@ parse_xml_tree (const xmlDocPtr parser_, string error_msg_)
 	int      cloaded;			// count of cards loaded
 	CardRef* card_ref;
 	bool     reversed;			// Is it a backward-oriented card?
-	size_t   capacity;
 	string   errmsg;
-	int      j;
+	u_int    j;
+
 	Glib::RefPtr<Gdk::Pixbuf> exp_light;
 	DeckPlayer* deck_player;
 
@@ -389,7 +388,9 @@ parse_xml_tree (const xmlDocPtr parser_, string error_msg_)
 	 */
 	j = m_sched_db.capacity () - CARDBOXES_MIN;
 	DL ((GRAPP,"Adding %d additional CardDeck%s\n", j, (j==1 ? ".":"s.")));
-	if (j > 0) {
+
+	if (j > 0) 
+	{
 		for (; j > 0; j--) {
 			m_card_deck.push_back (new CardDeck);
 		}
@@ -399,7 +400,7 @@ parse_xml_tree (const xmlDocPtr parser_, string error_msg_)
 	}
 	DL ((GRAPP,"m_card_deck.size () = %d\n", m_card_deck.size ()));
 		
-	for (int i = 0; i < m_sched_db.capacity (); i++) 
+	for (u_int i = 0; i < m_sched_db.capacity (); i++) 
 	{
 		DL((GRAPP,"Filling Card Deck #%d ...\n", i+1));
 		m_card_deck [i]->set_name (i);
@@ -572,7 +573,7 @@ save ()
 
 	/*= Write out decks
 	 */
-	for (int i = 0; i < m_sched_db.capacity (); i++) 
+	for (u_int i = 0; i < m_sched_db.capacity (); i++) 
 	{
 		ret = xmlTextWriterStartElement (writer, BAD_CAST "carddeck");
 		ret = xmlTextWriterWriteAttribute (writer, BAD_CAST "side",
@@ -642,7 +643,8 @@ close ()
 {
 	trace_with_mask("CardBox::close",GUITRACE);
 
-	int i;
+	u_int i;
+
 	for (i = 0; i < CARDBOXES_MIN; i++) 
 	{
 		DL ((DECK,"Resetting CardDeck #%d\n", i+1));
@@ -724,7 +726,7 @@ add_deck_to_box (int idx_, Deck* from_deck_)
 
 	bool with_reverse;
 
-	Assure_exit (idx_ < m_sched_db.capacity ());
+	Assure_exit ((u_int) idx_ < m_sched_db.capacity ());
 	Assure_exit (from_deck_);
 
 	if (m_fname.empty ()) {
@@ -899,7 +901,7 @@ move_card_to_box (VDeck::cardlist_iterator card_ref_,
 {
 	trace_with_mask("CardBox::move_card_to_box",GUITRACE);
 
-	int to_deck = 0;
+	u_int to_deck = 0;
 	bool is_expired = (dynamic_cast<CardRef*> (*card_ref_))->is_expired ();
 
 	/** Get hold of CardRef itself
@@ -939,7 +941,12 @@ move_card_to_box (VDeck::cardlist_iterator card_ref_,
 	 */
 	vcard->reschedule (answer_quality_);
 
-	if (answer_quality_ != ANSWER_BAD_BLACKOUT) {
+	/** TODO: I don't remember how different answers are handled
+	 */
+	if (answer_quality_ == ANSWER_OK ||
+		answer_quality_ == ANSWER_OK_HESITANT ||
+		answer_quality_ == ANSWER_OK_DIFFICULT)
+	{
 		to_deck = m_sched_db.find_next_box (vcard->get_interval ());
 	}
 	else {
@@ -955,20 +962,22 @@ move_card_to_box (VDeck::cardlist_iterator card_ref_,
 	 *  and the last deck for successful answer), the card goes back 
 	 *  to the end of the CardDeck.
 	 */
-	DL((GRAPP,"Placing CardRef with interval=%d to CardDeck #%d\n",
-		vcard->get_interval (), to_deck+1));
+	DL((GRAPP,"%s; moving Card \"%s\" (interval=%d, Deck %d->%d)\n",
+		GRANULE->quality_answer_str (answer_quality_),
+		vcard->get_question ().c_str (), 
+		vcard->get_interval (), from_deck_, to_deck));
 
 	/** TODO: Perhaps I should insert in the right place instead.
 	 */
 	m_card_deck [to_deck]->push_back (vcard);
 
 	if (GRANULE->dump_cardboxes_enabled ()) {
-		for (int j = 0; j < m_sched_db.capacity (); j++) {
+		for (u_int j = 0; j < m_sched_db.capacity (); j++) {
 			m_card_deck [j]->dump_to_file ();
 		}
 	}
 
-	if (to_deck != from_deck_) {
+	if (to_deck != (u_int) from_deck_) {
 		m_deck_player [from_deck_]->reset ();
 		m_card_deck [to_deck]->reset_progress ();
 	}
@@ -1121,7 +1130,7 @@ remove_all_duplicates (int idx_)
 
 	if (CONFIG->get_remove_duplicates ()) 
 	{
-		for (int idx = 0; idx < m_sched_db.capacity (); idx++) {
+		for (u_int idx = 0; idx < m_sched_db.capacity (); idx++) {
 			m_card_deck [idx]->remove_duplicates ();
 		}	
 	}
@@ -1140,7 +1149,7 @@ find_in_all_carddecks (CardRef* new_card_)
 	DL ((GRAPP,"Searching for \"%s\" in all CardDecks.\n",
 		 new_card_->get_question ().c_str ()));
 
-	for (int idx = 0; idx < m_sched_db.capacity (); idx++) 
+	for (u_int idx = 0; idx < m_sched_db.capacity (); idx++) 
 	{
 		iter = m_card_deck [idx]->begin ();
 		while (iter != m_card_deck [idx]->end ()) 
@@ -1174,9 +1183,7 @@ adjust_cardbox_tray ()
 {
 	trace_with_mask("CardBox::adjust_cardbox_tray",GUITRACE);
 
-	int i;
-	int j;
-	int diff;
+	u_int diff;
 	
 	diff = m_sched_db.capacity () - m_card_deck.size ();
 
@@ -1184,7 +1191,7 @@ adjust_cardbox_tray ()
 	 */
 	if (diff > 0) 
 	{	
-		for (i = m_card_deck.size () ; i < m_sched_db.capacity (); i++) 
+		for (u_int i = m_card_deck.size () ; i < m_sched_db.capacity (); i++) 
 		{
 			m_card_deck.push_back (new CardDeck);
 			m_card_deck [i]->set_name (i);
@@ -1204,7 +1211,7 @@ adjust_cardbox_tray ()
 		/*= Move all cards from CardDecks that are slated
 		 *  for removal to the last CardDeck of the new capacity.
 		 */
-		for (int j = new_sz; j < old_sz; j++) 
+		for (u_int j = new_sz; j < old_sz; j++) 
 		{
 			CardDeck::cardlist_const_iterator iter = m_card_deck [j]->begin ();
 			while (iter != m_card_deck [j]->end ()) 
@@ -1243,11 +1250,10 @@ normalize_cards ()
 	CardDeck* src_deck;
 	CardDeck* dest_deck;
 
-	int dest;
-	int src;
-	int i;
+	u_int i;
+	u_int dest;
 
-	for (src = 0; src < m_card_deck.size (); ++src) 
+	for (u_int src = 0; src < m_card_deck.size (); ++src) 
 	{
 		src_deck = m_card_deck [src];
 
@@ -1273,8 +1279,6 @@ normalize_cards ()
 
 	for (i = 0; i < m_card_deck.size (); ++i) 
 	{
-		/** TODO: update count of expired cards in the CardDeck.
-		 */
 		update_count (i);
 		m_card_deck [i]->reset_progress ();
 		m_deck_player [i]->reset ();
