@@ -2,7 +2,7 @@
 //------------------------------------------------------------------------------
 //                              CardDeck.cpp
 //------------------------------------------------------------------------------
-// $Id: CardDeck.cpp,v 1.30 2008/01/06 16:51:09 vlg Exp $
+// $Id: CardDeck.cpp,v 1.32 2008/08/04 02:54:31 vlg Exp $
 //
 //  Copyright (c) 2004,2006 by Vladislav Grinchenko
 //
@@ -58,9 +58,15 @@ void
 CardDeck::
 play_card (SideSelection side_, VDeck::cardlist_iterator& iter_)
 {
-	CardRef* cref = dynamic_cast<CardRef*>(*iter_);
+	CardRef* cref = dynamic_cast<CardRef*> (*iter_);
 	Assure_exit (cref);
-	cref->get_deck ().play_card (side_, iter_);
+
+	if (!Deck::audio_playback_active ()) {
+		cref->get_deck ().play_card (side_, iter_);
+	}
+	else {
+		DL ((GRAPP,"Already playing an audio clip.\n"));
+	}
 }
 
 void 
@@ -254,7 +260,7 @@ export_sound_bits (const std::string& path_)
 {
 	trace_with_mask("CardDeck::export_sound_bits",GUITRACE);
 
-#if !defined (WIN32)
+#if !defined (IS_WIN32)
 	string word;
 	string to;
 	string from;
@@ -281,7 +287,7 @@ export_sound_bits (const std::string& path_)
 		to = path_ + G_DIR_SEPARATOR_S + word[0];
 		if (!g_file_test (to.c_str (), G_FILE_TEST_IS_DIR)) 
 		{
-#if defined (WIN32)
+#if defined (IS_WIN32)
 			::mkdir (to.c_str ());
 #else
 			::mkdir (to.c_str (), 0755);
@@ -301,5 +307,5 @@ export_sound_bits (const std::string& path_)
 
 		citer++;
 	}
-#endif	// !def WIN32
+#endif	// !def IS_WIN32
 }
