@@ -2,7 +2,7 @@
 //------------------------------------------------------------------------------
 //                              CardDeck.h
 //------------------------------------------------------------------------------
-// $Id: CardDeck.h,v 1.21 2006/12/18 03:33:14 vlg Exp $
+// $Id: CardDeck.h,v 1.24 2008/01/06 16:51:09 vlg Exp $
 //
 //  Copyright (c) 2004,2006 by Vladislav Grinchenko
 //
@@ -22,12 +22,15 @@ using ASSA::ASSAERR;
 
 class VCard;
 
-/** To remove duplicates we compare Card References by the IDs of 
-	the cards they point to. If two cards happen to have the same
-	ID (case of reversed *sticky* duplicates), we arbitrarily
-	nominate lefthand-side be smaller then righthand-side.
-*/
-class Compare_CardRefs {
+/** @class Compare_CardRefs
+ *
+ * To remove duplicates we compare Card References by the IDs of 
+ * the cards they point to. If two cards happen to have the same
+ * ID (case of reversed *sticky* duplicates), we arbitrarily
+ * nominate lefthand-side be smaller then righthand-side.
+ */
+class Compare_CardRefs 
+{
 public:
 	int operator () (VCard* const& vc1_, VCard* const& vc2_)
 	{
@@ -50,6 +53,8 @@ public:
 };
 
 /** 
+ * @class CardDeck
+ *
  * CardDeck represents each of the Card Boxes on the left of the MainWindow.
  * Each CardDeck holds a list of virtual cards, CardRef, each of which points
  * to the real Card in a real Deck.
@@ -75,12 +80,12 @@ public:
 	virtual void push_back (VCard* card_);
 
 	/** Erase card from the VCard list only. Calling this method leaves 
-		the Deck intact.
-	*/
+	 *  the Deck intact.
+	 */
 	virtual void erase (cardlist_iterator iter_);
 
 	/** Erase card from the VCard list only. Calling this method leaves
-		the Deck intact.
+	 *	the Deck intact.
 	 */
 	virtual bool erase (VCard* card_);
 		
@@ -89,18 +94,18 @@ public:
 	void clear ();
 
 	/** Sort Card References based on their expiration time.
-		Those with oldest expiration dates go to the front of the list.
-	*/
+	 *  Those with oldest expiration dates go to the front of the list.
+	 */
 	void sort_by_expiration_time ();
 
-	virtual SideSelection get_side_selection ()    { return m_side_selection; }
+	virtual SideSelection get_side_selection () { return m_side_selection; }
 	virtual void set_side_selection (SideSelection s_) { m_side_selection=s_; }
 
 	virtual void play_card (SideSelection s_, VDeck::cardlist_iterator& iter_);
 
 	virtual const string&  get_name () const { return m_cdname; }
+	virtual int  get_index () const { return m_index; }
 
-	virtual int  get_index      () const;
 	virtual void get_progress   (float& pct_done_, std::string& msg_, int idx_);
 	virtual void reset_progress ();
 
@@ -108,8 +113,8 @@ public:
 	void remove_duplicates (); 
 
 	/** Shuffle all subsets with equal expiration 
-		timestamp, preserving the general expiration order.
-	*/
+	 *  timestamp, preserving the general expiration order.
+	 */
 	void shuffle_subsets ();
 
 	/** Dump the content of the deck to file (for troubleshooting).
@@ -121,23 +126,33 @@ public:
 	void export_sound_bits (const std::string& path_);
 
 private:
-	string         m_cdname;	// Number of the card box this deck represents
+    /// Number of the card box this deck represents
+	string m_cdname;	
+
+	/// CardDeck index 
+	int m_index;
+
+	/// A collection of cards
 	cardlist_type  m_cards;
 
     /// The size at the moment when DeckPlayer started playing the Deck.
-	u_int          m_init_size;	
+	u_int m_init_size;	
 
 	/// How many cards has been already played by the DeckPlayer so far.
-	u_int          m_progress_idx;
+	u_int m_progress_idx;
 };
 
-//------Inlines ---------------------------------------------------------------
+/**-----------------------------------------------------------------------------
+ * Inlined methods
+ **-----------------------------------------------------------------------------
+ */
 inline
 CardDeck::
 CardDeck () 
 	: 
-	m_init_size      (0), 
-	m_progress_idx   (0)
+	m_index (0),
+	m_init_size (0), 
+	m_progress_idx (0)
 {
 	trace_with_mask("CardDeck::CardDeck",GUITRACE|DECK);
 }
@@ -156,15 +171,6 @@ push_back (VCard* card_)
 	m_cards.push_back (card_); 
 }
 
-inline void
-CardDeck::
-reset_progress () 
-{ 
-	trace_with_mask("CardDeck::reset_progress",GUITRACE|DECK);
-	m_init_size = size ();
-	m_progress_idx = 0;
-}
-
 inline void 
 CardDeck::
 erase (cardlist_iterator iter_) 
