@@ -2,7 +2,7 @@
 //------------------------------------------------------------------------------
 //                              CardRef.cpp
 //------------------------------------------------------------------------------
-// $Id: CardRef.cpp,v 1.14 2008/01/06 16:51:09 vlg Exp $
+// $Id: CardRef.cpp,v 1.15 2008/05/19 03:39:03 vlg Exp $
 //
 //  Copyright (c) 2004 by Vladislav Grinchenko
 //
@@ -282,12 +282,10 @@ is_expired ()
 }
 
 /**
- * Adjust card's E-Factor base on the quality of answer.
+ * Adjust E-Factor of the Card base on the quality of answer.
  * Adjust repetition counter.
  * Calculate new repetition interval.
  * Adjust expiration date.
- *
- * This version only covers Bad/Good (but not perfect) answer.
  */
 void
 CardRef::
@@ -298,17 +296,19 @@ reschedule (AnswerQuality answer_quality_)
 	dump_schedule ("Before adjustment");
 	DL ((DECK,"answer_quality_ = %d\n", answer_quality_));
 
-	if (answer_quality_ == ANSWER_OK_HESITANT) 
+	if (answer_quality_ == ANSWER_OK || answer_quality_ == ANSWER_OK_HESITANT)
 	{
-// This is for ANSWER_OK. For OK_HESITANT, don't change EFactor.
-//
-//		float ef = m_efactor + 0.1;
-//		if (fabsf (ef - ScheduleDB::EF_MAX) < 0.01) {
-//			m_efactor = ScheduleDB::EF_MAX;
-//		}
-//		else {
-//			m_efactor = ef;
-//		}
+		if (answer_quality_ == ANSWER_OK) {
+			float ef = m_efactor + 0.1;
+			if (fabsf (ef - ScheduleDB::EF_MAX) < 0.01) {
+				m_efactor = ScheduleDB::EF_MAX;
+			}
+			else {
+				/** For OK_HESITANT, EFactor is not changed.
+				 */
+				m_efactor = ef;
+			}
+		}
 		
 		m_repetition++;
 		if (m_repetition == 1) {
@@ -318,9 +318,8 @@ reschedule (AnswerQuality answer_quality_)
 			m_interval = int (m_interval * m_efactor);
 		}
 	}
-	else  /* ANSWER_BAD_BLACKOUT */
+	else  /* The rest, including ANSWER_OK_DIFFICULT - keep EFactor */
 	{
-		/** Keep EFactor */
 		m_repetition = 0;
 		m_interval = ScheduleDB::BOX_ONE_STEP;
 	}
