@@ -2,7 +2,7 @@
 //------------------------------------------------------------------------------
 //                              CardView.cpp
 //------------------------------------------------------------------------------
-// $Id: CardView.cpp,v 1.61 2008/01/14 03:41:00 vlg Exp $
+// $Id: CardView.cpp,v 1.66 2008/08/19 02:52:46 vlg Exp $
 //
 //  Copyright (c) Vladislav Grinchenko
 //
@@ -129,12 +129,13 @@ CardView (VCard* card_)
 	m_response      (Gtk::RESPONSE_OK),
 
 	m_cancel_button (NULL),
-	m_count_button  (NULL),
 	m_add_n_close_button (NULL),
 	m_add_button    (NULL),
+	m_count_button  (NULL),
 	m_help_button   (NULL),
 
 	m_front_text    (NULL),
+	m_img_text      (NULL),
 	m_back_text     (NULL),
 	m_example_text  (NULL),
 	m_example_img_text (NULL),
@@ -142,7 +143,8 @@ CardView (VCard* card_)
 	m_asb_text      (NULL),
 
 	m_edit_controls (NULL),
-	m_help_dialog   (NULL)
+	m_help_dialog   (NULL),
+	m_cards_count   (0)
 {
 	trace_with_mask("CardView::CardView(VCard&)",GUITRACE);
 
@@ -330,7 +332,7 @@ CardView (VCard* card_)
 	m_img_text->set_buffer (m_textbuf_ref [TV_IMG]);
 
 	m_textbuf_ref [TV_EIMG] = Gtk::TextBuffer::create();
-	m_example_img_text->set_buffer (m_textbuf_ref [TV_IMG]);
+	m_example_img_text->set_buffer (m_textbuf_ref [TV_EIMG]);
 #endif
 
 	/**----------------**
@@ -489,15 +491,8 @@ CardView (VCard* card_)
 									   &EditControls::on_focus_changed),
 							  m_example_img_text));
 
-#ifdef IS_HILDON
-	/** Trap 'on virtual keyboard close' event to restore
-	 *  original size of the dialog
-	 */
-#ifdef HILDON_DEPRICATED
-	HILDONAPPWIN->signal_im_close ().connect (
-		mem_fun (*this, &CardView::hildon_im_close_cb));
-#endif
-#endif
+ 	signal_delete_event ().connect (
+ 		sigc::mem_fun (*this, &CardView::on_delete_window_clicked));
 
 	/*********************************************************************
 	 *  Table holds elements together:
@@ -725,7 +720,11 @@ run_in_edit_mode (Gtk::Window& parent_, bool disable_cancel_)
 	take_snapshot ();
 
 	GRANULE->move_child_to_parent_position (parent_, *this);
+
+	show ();
+	present ();
 	gtk_main ();
+
 	GRANULE->move_parent_to_child_position (parent_, *this);
 
 	return (m_syntax_error ? Gtk::RESPONSE_REJECT : m_response);
@@ -735,7 +734,9 @@ run_in_edit_mode (Gtk::Window& parent_, bool disable_cancel_)
  * Handling the very first card is different from the rest.
  *
  * The first card is created by the DeckView.
- * If it is cancelled, we are responsible for memory deallocation.
+ *
+ * If card creation is cancelled by the user, 
+ * we are responsible for memory deallocation.
  *
  * The rest of the cards are created here.
  *
@@ -745,9 +746,9 @@ run_in_edit_mode (Gtk::Window& parent_, bool disable_cancel_)
  *
  * Possible values returned by run():
  *
- *    <Cancel> - Gtk::RESPONSE_CANCEL
- *    <Add>    - Gtk::RESPONSE_APPLY
- *    <Ok>     - Gtk::RESPONSE_OK
+ *    <Cancel> - Gtk::RESPONSE_CANCEL : cancel dialog
+ *    <Add>    - Gtk::RESPONSE_APPLY  : add new card and keep dialog open
+ *    <Ok>     - Gtk::RESPONSE_OK     : add new card and close dialog
  **/
 int
 CardView::
@@ -760,22 +761,28 @@ run_in_add_mode (DeckView& deckview_, AutoFillDB& autofill_db_)
 	refresh_cards_count (deckview_);
 	apply_patterns (autofill_db_, deckview_.get_deck_size ()+1);
 
+	/** Show the dialog for the first card.
+	 */
 	GRANULE->move_child_to_parent_position (deckview_, *this);
-	gtk_main ();
 
-	if (m_syntax_error) 
+	while (true) 
 	{
+		show ();
+		present ();
+		gtk_main ();
+
 		DL ((GRAPP,"Caught syntax error : RESPONSE_REJECT\n"));
-		m_response = Gtk::RESPONSE_REJECT;
-		delete m_card;
-		goto done;
-	}
+		if (m_response == Gtk::RESPONSE_CANCEL) 
+		{
+			DL ((GRAPP,"Dialog was cancelled - RESPONSE_CANCEL\n"));
+			delete m_card;
+			goto done;
+		}
 
-	if (m_response == Gtk::RESPONSE_CANCEL) 
-	{
-		DL ((GRAPP,"Dialog cancelled : RESPONSE_CANCEL\n"));
-		delete m_card;
-		goto done;
+		if (m_syntax_error == false) 
+		{
+			break;
+		}
 	}
 
 	deckview_.add_new_card (dynamic_cast<Card*>(m_card));
@@ -789,13 +796,14 @@ run_in_add_mode (DeckView& deckview_, AutoFillDB& autofill_db_)
 		refresh_cards_count (deckview_);
 		apply_patterns (autofill_db_, deckview_.get_deck_size ()+1);
 
+		show ();
+		present ();
 		gtk_main ();
 
 		if (m_syntax_error) {
-			delete m_card;
-			m_response = Gtk::RESPONSE_REJECT;
-			break;
+			continue;			// give the user a chance to fix errors
 		}
+
 		if (m_response == Gtk::RESPONSE_CANCEL) {
 			delete m_card;
 			break;
@@ -816,7 +824,9 @@ on_add_clicked  ()
 	trace_with_mask("CardView::on_add_clicked",GUITRACE);
 
 	on_addnclose_clicked ();
-	clear_dialog ();
+	if (!m_syntax_error) {
+		clear_dialog ();
+	}
 	m_keep_open = true;
 	m_response = Gtk::RESPONSE_APPLY;
 }
@@ -827,17 +837,20 @@ on_addnclose_clicked  ()
 {
 	trace_with_mask("CardView::on_addnclose_clicked",GUITRACE);
 
+	Glib::ustring tmp;
+	Glib::ustring text = m_front_text->get_buffer ()->get_text ();
+
+	m_syntax_error = false;
 	m_response = Gtk::RESPONSE_OK;
 
-	Glib::ustring text = m_front_text->get_buffer ()->get_text ();
 
 	if (text.length () == 0) 
 	{
 		Gtk::MessageDialog err0 (
-	_("<b>Front:</b> text field <span foreground=\"red\">IS EMPTY</span>!\n\n"
+	_("<b>Front</b> field <span foreground=\"red\">IS EMPTY</span>!\n\n"
 	  "This is an illegal syntax.\n"
 	  "You must fill it with (unique) text.\n"),
-	true, Gtk::MESSAGE_ERROR);
+	true, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_CLOSE);
 		err0.run ();
 		m_syntax_error = true;
 		return;
@@ -846,8 +859,8 @@ on_addnclose_clicked  ()
 	if (!Granule::check_markup (text)) 
 	{
 		Gtk::MessageDialog err1 (
-			_("FRONT field contains illegal markup tag(s)!"),
-			false, Gtk::MESSAGE_ERROR);
+			_("<b>Front</b> field contains illegal markup tag(s)!"),
+			true, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_CLOSE);
 		err1.run ();
 		m_syntax_error = true;
 		return;
@@ -856,8 +869,8 @@ on_addnclose_clicked  ()
 	if (!Granule::check_markup (m_back_text->get_buffer ()->get_text ())) 
 	{
 		Gtk::MessageDialog err2 (
-			_("BACK field contains illegal markup tag(s)!"),
-			false, Gtk::MESSAGE_ERROR);
+			_("<b>Back</b> field contains illegal markup tag(s)!"),
+			true, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_CLOSE);
 		err2.run ();
 		m_syntax_error = true;
 		return;
@@ -866,20 +879,46 @@ on_addnclose_clicked  ()
 	if (!Granule::check_markup (m_example_text->get_buffer ()->get_text ())) 
 	{
 		Gtk::MessageDialog err3 (
-			_("EXAMPLE field contains illegal markup tag(s)!"),
-			false, Gtk::MESSAGE_ERROR);
+			_("<b>Example</b> field contains illegal markup tag(s)!"),
+			true, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_CLOSE);
 		err3.run ();
 		m_syntax_error = true;
 		return;
 	}
+	
+	text = m_img_text->get_buffer ()->get_text ();
+	if (text.length () > 0) {
+		if (!Granule::test_file_exists (text, m_card->get_deck_path (), tmp))
+		{
+			Gtk::MessageDialog err4 (
+				_("<b>Front IMG</b> field contains non-existent filename!"),
+				true, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_CLOSE);
+			err4.run ();
+			m_syntax_error = true;
+			return;
+		}
+	}
+
+	text = m_example_img_text->get_buffer ()->get_text ();
+	if (text.length () > 0) {
+		if (!Granule::test_file_exists (text, m_card->get_deck_path (), tmp))
+		{
+			Gtk::MessageDialog err5 (
+				_("<b>Example IMG</b> field contains non-existent filename!"),
+				true, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_CLOSE);
+			err5.run ();
+			m_syntax_error = true;
+			return;
+		}
+	}
 
 	/** Transfer data from the widget to the Card object
 	 */
 	if (m_card != NULL && text_modified ()) 
 	{
-		m_card->set_question     (m_front_text   ->get_buffer ()->get_text ());
-		m_card->set_answer       (m_back_text    ->get_buffer ()->get_text ());
-		m_card->set_example      (m_example_text ->get_buffer ()->get_text ());
+		m_card->set_question (m_front_text  ->get_buffer ()->get_text ());
+		m_card->set_answer   (m_back_text   ->get_buffer ()->get_text ());
+		m_card->set_example  (m_example_text->get_buffer ()->get_text ());
 
 		m_card->set_alt_spelling (ASF, m_asf_text->get_buffer()->get_text ());
 		m_card->set_alt_spelling (ASB, m_asb_text->get_buffer()->get_text ());
@@ -893,9 +932,23 @@ on_addnclose_clicked  ()
 	m_keep_open = false;
 	m_front_text->grab_focus ();
 
+	hide ();
 	gtk_main_quit ();
 }
 
+/**
+ * @return true - top other handlers from being invoked for the event.
+ */
+bool
+CardView::
+on_delete_window_clicked (GdkEventAny* /*event_*/)
+{
+    trace_with_mask("CardView::on_delete_window_clicked",GUITRACE);
+
+	on_cancel_clicked ();
+    return true;  
+}
+
 void
 CardView::
 on_cancel_clicked  ()
@@ -903,6 +956,7 @@ on_cancel_clicked  ()
 	trace_with_mask("CardView::on_cancel_clicked",GUITRACE);
 	
 	m_response = Gtk::RESPONSE_CANCEL;
+	hide ();
 	gtk_main_quit ();
 }
 
@@ -950,11 +1004,11 @@ on_key_press_event (GdkEventKey* event_)
  *  + 'image_path' (RO) is absolute path of the Deck the card came from.
  *  + 'image' (RW) is the relative path + filename.
  *
- * We show 'image' property below its corresponding the m_img_text entry widget.
+ * We show 'image' property below its corresponding m_img_text entry widget.
  * If user clicks on <Browse>, the file chooser changes to 'image_path/image'.
  *
  * When user clicks on <Browse>, we take the new image pathname, and 
- * calcuate its path relevant to the 'image_path' (deck path), and
+ * calcuate its path relative to the 'image_path' (deck path), and
  * set the result to m_img_text. 
  *
  * The 'image' part is saved in the deck XML file as an attribute of
@@ -966,11 +1020,20 @@ on_browse_img_clicked  (int type_)
 {
 	trace_with_mask("CardView::on_browse_img_clicked",GUITRACE);
 
+	PathMode path_mode = m_card->image_path_mode ();
+
+	DL ((GRAPP,"Image path mode: %s\n",
+		 (path_mode == PATH_RELATIVE ? "relative" : "absolute")));
+
 	FileOpenDialog f (_("Select Image File"), this);
 
 	string selection;
 	Glib::ustring path;
 	
+	/**
+	 * Set up initial path to guide FileOpenDialog
+	 * to the right place in the filesystem.
+	 */
 	if (type_ == TV_IMG) {
 		path= m_img_text->get_buffer ()->get_text ();
 	}
@@ -981,7 +1044,7 @@ on_browse_img_clicked  (int type_)
 	if (!path.empty ()) 
 	{
 		selection = Granule::filename_from_utf8 (path);
-		if (m_card->image_path_mode () == PATH_RELATIVE) 
+		if (path_mode == PATH_RELATIVE) 
 		{
 			if (!g_path_is_absolute (selection.c_str ())) {
 				path = m_card->get_deck_path () + G_DIR_SEPARATOR_S + path;
@@ -997,36 +1060,56 @@ on_browse_img_clicked  (int type_)
                     m_card->get_deck_path (), selection);
             }
 		}
-
-		DL ((GRAPP,"calling FileOpenDialog (\"%s\")\n", path.c_str ()));
+		DL ((GRAPP,"Set initial pathname to \"%s\"\n", path.c_str ()));
 		f.set_filename (path);
 	}
 
-    if (f.run () == Gtk::RESPONSE_OK) 
+	DL ((GRAPP,"calling FileOpenDialog (\"%s\")\n", path.c_str ()));
+
+    if (f.run () != Gtk::RESPONSE_OK)
 	{
-        selection = Granule::filename_from_utf8 (f.get_filename ());
+		return;
+	}
 
-		if (m_card->image_path_mode () == PATH_RELATIVE) 
-        {
-            string rp = Granule::calculate_relative_path (
-                m_card->get_deck_path (), selection);
+	selection = Granule::filename_from_utf8 (f.get_filename ());
 
-			if (type_ == TV_IMG) {
-				m_img_text->get_buffer ()->set_text (rp);
-			}
-			else {
-				m_example_img_text->get_buffer ()->set_text (rp);
-			}
-        }
-        else {
-			if (type_ == TV_IMG) {
-				m_img_text->get_buffer ()->set_text (selection);
-			}
-			else {
-				m_example_img_text->get_buffer ()->set_text (selection);
-			}
-        }
-    }
+	if (selection.length () == 0) {
+		const Glib::ustring msg ("The filepath \"" + f.get_filename () +
+								 " contains characters not representable " +
+								 "in the encoding of the local filesystem.");
+		Gtk::MessageDialog error_dialog (*this, msg, false, 
+										 Gtk::MESSAGE_ERROR, 
+										 Gtk::BUTTONS_OK, 
+										 true);
+		error_dialog.run();
+		return;
+	}
+
+	if (path_mode == PATH_RELATIVE) 
+	{
+		DL ((GRAPP,"Deck absolute path: '%s'\n", 
+			 m_card->get_deck_path ().c_str ()));
+
+		string rp = Granule::calculate_relative_path (
+			m_card->get_deck_path (), selection);
+
+		DL ((GRAPP,"Relative path calculated: '%s'\n", rp.c_str ()));
+
+		if (type_ == TV_IMG) {
+			m_img_text->get_buffer ()->set_text (rp);
+		}
+		else {
+			m_example_img_text->get_buffer ()->set_text (rp);
+		}
+	}
+	else {
+		if (type_ == TV_IMG) {
+			m_img_text->get_buffer ()->set_text (selection);
+		}
+		else {
+			m_example_img_text->get_buffer ()->set_text (selection);
+		}
+	}
 }
 
 void
@@ -1073,7 +1156,9 @@ on_help_clicked  ()
 	if (m_help_dialog == NULL) {
 		m_help_dialog = new CardViewHelpDialog (*this);
 	}
+
 	m_help_dialog->show ();
+	m_help_dialog->present ();
 }
 
 void
@@ -1107,17 +1192,6 @@ size_allocate_cb (Gtk::Allocation& allocation_)
 									allocation_.get_height ());
 }
 
-#ifdef IS_HILDON
-void
-CardView::
-hildon_im_close_cb ()
-{
-    trace_with_mask("CardView(hildon)::hildon_im_close_cb",GUITRACE);
-
-	set_size_request (672, 396);
-}
-#endif
-
 void
 CardView::
 refresh_cards_count (DeckView& deckview_)
