@@ -2,7 +2,7 @@
 //------------------------------------------------------------------------------
 //                              CardView.h
 //------------------------------------------------------------------------------
-// $Id: CardView.h,v 1.30 2008/02/04 04:02:20 vlg Exp $
+// $Id: CardView.h,v 1.32 2008/06/02 03:03:53 vlg Exp $
 //
 //  Copyright (c) Vladislav Grinchenko
 //
@@ -39,7 +39,14 @@ class AutoFillDB;
 class ButtonWithImageLabel;
 
 enum { 
-	TV_FRONT, TV_BACK, TV_EXAMPLE, TV_ASF, TV_ASB, TV_IMG, TV_EIMG, TV_SZ 
+	TV_FRONT, 
+	TV_BACK, 
+	TV_EXAMPLE, 
+	TV_ASF, 
+	TV_ASB, 
+	TV_IMG, 
+	TV_EIMG, 
+	TV_SZ 
 };
 
 /** @class CardView
@@ -77,11 +84,7 @@ public:
 
 	/** We block [x] delete action and reroute it to <Close>
 	 */
-	virtual bool on_delete_event (GdkEventAny* event_) 
-	{
-		on_cancel_clicked ();
-		return true;
-	}
+	bool on_delete_window_clicked (GdkEventAny* event_);
 
 	/** Remember window size when resized.
 	 */
@@ -126,7 +129,7 @@ private:
 	ButtonWithImageLabel* m_add_button;
 
 	Gtk::Button*    m_count_button;	          // functions as a label
-	ButtonWithImageLabel*    m_help_button;
+	ButtonWithImageLabel* m_help_button;
 
 	Gtk::Button*    m_browse_img_button;  // Select Front Image 
 	Gtk::Button*    m_browse_example_img_button;  // Select Example Image 
@@ -146,7 +149,7 @@ private:
 	/** Temporal fix - in hildon, Gtk::TextView doesn't allocate 
 	 *  default buffer.
 	 */ 
-	Glib::RefPtr<Gtk::TextBuffer> m_textbuf_ref[TV_SZ];
+	Glib::RefPtr<Gtk::TextBuffer> m_textbuf_ref [TV_SZ];
 #endif
 
 	// -*- Snapshot to detect true changes -*-
