@@ -2,7 +2,7 @@
 //------------------------------------------------------------------------------
 //                              Deck.cpp
 //------------------------------------------------------------------------------
-// $Id: Deck.cpp,v 1.74 2008/02/04 04:02:20 vlg Exp $
+// $Id: Deck.cpp,v 1.82 2008/08/19 02:52:46 vlg Exp $
 //
 //  Copyright (c) Vladislav Grinchenko
 //
@@ -21,13 +21,15 @@ using std::ifstream;
 #include <glib.h>
 #include <glib/gstdio.h>
 
-#if !defined (WIN32)
+#if !defined (IS_WIN32)
 #  include <libgen.h>               // basename(3), dirname(3)
+#  include <fcntl.h>
 #endif
 
 #include <assa/CommonUtils.h>
 #include <assa/Fork.h>
 #include <assa/TimeVal.h>
+#include <assa/AutoPtr.h>
 
 #include "Granule.h"
 #include "Granule-main.h"
@@ -55,22 +57,35 @@ public:
 	}
 };	
 
+FILE* Deck::s_pipe_stream = NULL;
+
+void
+Deck::
+init_internals ()
+{
+	m_snd_enabled       = false;
+	m_snd_path_mode     = PATH_ABSOLUTE;
+	m_pics_path_mode    = PATH_RELATIVE;
+	m_side_selection    = FRONT;
+	m_custom_appearance = false;
+}
+
 /** C'tor for existing deck
  */
 Deck::
 Deck (DeckEditStatus status_) 
 	: 
-	m_side_selection (FRONT),
-	m_status         (status_), 
-	m_needs_renaming (false),
-	m_snd_enabled    (false),
-	m_snd_path_mode  (PATH_ABSOLUTE),
-	m_custom_appearance (false)
+	m_audio_playback_pipe (NULL),
+	m_status              (status_), 
+	m_needs_renaming      (false),
+	m_side_selection      (FRONT)
 {
 	trace_with_mask("Deck::Deck",GUITRACE|DECK);
 
-	m_active_snd_path = 
-		Granule::filename_to_utf8 (CONFIG->get_snd_archive_path ());
+	init_internals ();
+
+	m_active_snd_path = Granule::filename_to_utf8 (
+		CONFIG->get_snd_archive_path ());
 
 	m_text_colors_db.init_deck_defaults ();
 	m_fonts_db.init_deck_defaults ();
@@ -83,18 +98,26 @@ Deck::
 Deck (const string& fname_, DeckEditStatus status_) 
 	:
 	m_fname  (fname_), 
-	m_desc   ("Created on "),
 	m_status (status_),
 	m_needs_renaming ((status_ == NEW_DECK)),
+	m_desc   ("Created on "),
 	m_custom_appearance (false)
 {
 	trace_with_mask("Deck::Deck(untitled)",GUITRACE|DECK);
 
+	init_internals ();
+
 	m_author = Granule::locale_to_utf8 (CONFIG->get_user_name ());
 	m_active_snd_path = 
 		Granule::filename_to_utf8 (CONFIG->get_snd_archive_path ());
 
-	m_deck_path = dirname ((char*)fname_.c_str ());
+	/** It is very important to have valid absolute path name
+	 *  for a newly created Deck.
+	 */
+	m_deck_path = CONFIG->get_filesel_path ();
+	if (!Glib::path_is_absolute (m_deck_path)) {
+		m_deck_path = GRANULE->get_current_working_dir ();
+	}
 
 	m_text_colors_db.init_deck_defaults ();
 	m_fonts_db.init_deck_defaults ();
@@ -248,15 +271,16 @@ parse_xml_tree (xmlDocPtr parser_, std::string& error_msg_)
 	 */
 	test_active_snd_path ();
 
-	/** Fetch pics_path if present. If absent, assume absolute.
+	/** Fetch pics_path if present. Default is relative.
 	 */
-	m_pics_path_mode = PATH_ABSOLUTE;
-
 	result = Granule::get_node_by_xpath (parser_, "/deck/pics_path/@relative");
 	if (result) {
 		if (!strcmp ((const char*) result, "yes")) {
 			m_pics_path_mode = PATH_RELATIVE;
 		}
+		else {
+			m_pics_path_mode = PATH_ABSOLUTE;
+		}
 		xmlFree (result);
 	}
 
@@ -707,8 +731,8 @@ split_tokens (const Glib::ustring& instream_,
 {
     trace_with_mask("Deck::split_tokens",GUITRACE);
 
-	int pos_start = 0;
-	int pos_end = 0;
+	u_int pos_start = 0;
+	u_int pos_end = 0;
 	bool bracket_open = bracket_flag_;
 
 	tokens_.clear ();
@@ -720,7 +744,9 @@ split_tokens (const Glib::ustring& instream_,
 			DL ((CSVPARSER,"pos_start = '%c', pos_end = '%c'\n",
 				 instream_[pos_start], instream_[pos_end]));
 
-			if (instream_[pos_end] == separator_ && !bracket_open) {
+			if (instream_[pos_end] == (gunichar) separator_ && 
+				! bracket_open) 
+			{
 				DL ((CSVPARSER,"copying [%s]\n", 
 					 instream_.substr (pos_start, pos_end-pos_start).c_str ()));
 				tokens_.push_back (
@@ -747,11 +773,11 @@ split_tokens (const Glib::ustring& instream_,
 	else if (tokens_.size () == 2) { DL ((GRAPP,"Case #1 (2 tokens)\n")); }
 	else if (tokens_.size () == 3) { DL ((GRAPP,"Case #2 (3 tokens)\n")); }
 
-	for (int i=0; i < tokens_.size (); i++) {
+	for (u_int i = 0; i < tokens_.size (); i++) {
 		DL ((GRAPP,"token [%d] = [%s]\n", i, tokens_[i].c_str ()));
 	}
 
-	return (tokens_.size ()-1);
+	return (tokens_.size () - 1);
 }
 
 void
@@ -1326,237 +1352,6 @@ find_card_by_id (long id_)
 	return (NULL);
 }
 
-/** Rules for playable name are as follows:
- *
- *  <ol>
- *   <li> A word can be preceeded by "to " to indicate that it is a verb. </li>
- *
- *   <li> All multiword expressions are ignored. </li>
- *
- *	 <li> The WAV file is assumed to be stored in the directory
- *     with the name of its first letter, i.e. "granule" will be
- *     stored in the directory "/path/to/dictionary/g/", and thus 
- *     the full name would be "/path/to/dictionary/g/granule.wav".
- *   </li>
- *
- *	@return Playable name on success; an empty string on error.
- */
-void
-Deck::
-play_card (SideSelection side_, Deck::cardlist_iterator& iter_)
-{
-	trace_with_mask("Deck::play_card",GUITRACE|DECK);
-
-	static char execmdbuf [PATH_MAX];
-	std::vector<string> v;
-	string w;
-
-	if (!m_snd_enabled) {
-		DL ((GRAPP,"Sound is disabled!\n"));
-		return;
-	}
-
-	DL ((GRAPP,"play card for side '%s'\n", (side_==FRONT?"Front":"Back")));
-
-	w = Granule::locale_from_utf8 (
-		(*iter_)->get_alt_spelling (side_ == FRONT ? ASF : ASB));
-
-	if (w.length () == 0) 
-	{
-		w = Granule::locale_from_utf8 ((*iter_)->get_question ());
-		if (w.length () == 0) {
-			/* conversion failed (say, UTF-8 to ASCII)
-			 */
-			return;
-		}
-
-		ASSA::Utils::split (w.c_str (), v);
-		if (v.size () > 1) 
-		{
-			if (v[0] != "to") {
-				return;
-			}
-			w = v[1];
-		}
-	}
-	
-	int n;
-	const char* cmd = CONFIG->get_snd_player_cmd ();
-
-	string sound_fname (m_active_snd_path + G_DIR_SEPARATOR_S);
-	sound_fname += w[0];
-	sound_fname += G_DIR_SEPARATOR_S + w; 
-	
-	string cmd_args (CONFIG->get_snd_player_args ());
-	string file_ext;
-
-// FIXME:
-//   This would not work any more because the type of the file 
-//   (and thus the whole name) is specified in the command line argument.
-//   But, without this check, what follows might cause a core dump.
-//   So, you need to fix it somehow!
-//
-//	struct stat st;
-//	if (::stat (sound_fname.c_str (), &st) != 0 || !S_ISREG (st.st_mode)) {
-//		DL((APP,"Can't stat sound file \"%s\"\n", sound_fname.c_str ()));
-//		return;
-//	}
-
-	DL((APP,"snd filename=\"%s\"\n", sound_fname.c_str ()));
-
-    /**
-	 * Win32
-	 * -----
-	 * If sound player command is an empty string, then use
-	 * build-in system PlaySound() 
-	 *   (FIXME: test it if it really works. Looks like you still need
-	 *           the command arguments format to add the sound extension)
-	 *      
-	 * Otherwise, use the external command.
-	 */
-	/** Arguments composition common for all
-	 */
-	snprintf (execmdbuf, PATH_MAX, cmd_args.c_str (), sound_fname.c_str ());
-
-#if defined (WIN32)
-	if (cmd != NULL && strlen (cmd) != 0) 
-	{
-		n = snprintf (execmdbuf, PATH_MAX, "\"%s\" ", cmd);
-		sprintf(execmdbuf+n, PATH_MAX-n, cmd_args.c_str(), sound_fname.c_str());
-		DL((GRAPP,"win32: system(\"%s\")\n", execmdbuf));
-		system (execmdbuf);
-	}
-	else {
-		DL((GRAPP,"win32: PlaySound(\"%s\")\n", execmdbuf));
-		PlaySound (execmdbuf, NULL, SND_FILENAME| SND_ASYNC);
-	}
-    /**
-	 * Hildon
-	 * ------
-	 * If sound command is an empty string, use build-in playback
-	 * (this only works with WAV files and produces really bad quality 
-	 *  sound).
-	 *
-	 * Otherwise, use DPS to play mp3 files with no impact on main CPU
-	 * with this command:
-	 *
-	 *   gst-launch filesrc location=/path/to/your.mp3 ! dspmp3sink
-	 *
-	 * 
-	 */
-#elif defined (IS_HILDON)
-	if (cmd != NULL && strlen (cmd) != 0) 
-	{
-		DL((APP,"hildon: fork_exec(\"%s %s\")\n", cmd, execmdbuf));
-//		ASSA::Fork::fork_exec (cmd, execmdbuf, ASSA::Fork::IGNORE_STATUS);
-		n = ASSA::Fork::fork_exec (cmd, execmdbuf, ASSA::Fork::COLLECT_STATUS);
-		DL((GRAPP,"fork_exec() = %d (0:success, 1:failed, -1:wait(2) failed\n",
-			n));
-	}
-
-    /**
-	 * POSIX (Desktops)
-	 * ----------------
-	 * For now, we squarely rely on external command
-	 *
-	 * FIXME: for the future, evaluate gstreamer framework.
-	 *
-	 *
-	 */
-#else
-	/**	
-	 * NOTE: New Gtk 2.4 FileChooserDialog changes disposition mask for SIGCHLD.
-	 *       We have to preserve it - otherwise, we would crash and burn.
-	 *       COLLECT_STATUS option does that for us here.
-	*/
-	DL((APP,"posix: fork_exec(\"%s %s\")\n", cmd, execmdbuf));
-
-	ASSA::Fork::fork_exec (cmd, execmdbuf, ASSA::Fork::IGNORE_STATUS);
-#endif
-
-}
-
-// 	trace_with_mask("Deck::play_card",GUITRACE|DECK);
-
-// 	static char execmdbuf [1024];
-// 	std::vector<string> v;
-// 	string w;
-
-// 	if (!m_snd_enabled) {
-// 		DL ((GRAPP,"Sound is disabled!\n"));
-// 		return;
-// 	}
-
-// 	DL ((GRAPP,"play card for side '%s'\n", (side_==FRONT?"Front":"Back")));
-
-//	w = Granule::locale_from_utf8 ((*iter_)->get_alt_spelling (side_ == FRONT ? ASF : ASB));
-
-// 	if (w.length () == 0) 
-// 	{
-// 		w = Granule::locale_from_utf8 ((*iter_)->get_question ());
-// 		ASSA::Utils::split (w.c_str (), v);
-// 		if (v.size () > 1) 
-// 		{
-// 			if (v[0] != "to") {
-// 				return;
-// 			}
-// 			w = v[1];
-// 		}
-// 	}
-// 	string sfn (m_active_snd_path + G_DIR_SEPARATOR_S);
-// 	sfn += w[0];
-// 	sfn += G_DIR_SEPARATOR_S + w; /// + ".wav";
-
-//	struct stat st;
-//	if (::stat (sfn.c_str (), &st) != 0 || !S_ISREG (st.st_mode)) {
-//		DL((APP,"Can't stat sound file \"%s\"\n", sfn.c_str ()));
-//		return;
-//	}
-
-// 	DL((APP,"Play sound bite \"%s\"\n", sfn.c_str ()));
-
-// #if defined (WIN32)
-
-// 	int n;
-// 	const char* cmd = CONFIG->get_snd_player_cmd ();
-// 	if (cmd != NULL && strlen (cmd) != 0) {
-// 		n = sprintf (execmdbuf, "\"%s\" ", cmd);
-// 		sprintf (execmdbuf+n, CONFIG->get_snd_player_args (), sfn.c_str ());
-
-// 		DL((GRAPP,"Win32: playing \"%s %s\"\n", execmdbuf));
-// 		system (execmdbuf);
-// 	}
-// 	else {
-// 		PlaySound (sfn.c_str (), NULL, SND_FILENAME| SND_ASYNC);
-// 	}
-
-// #elif defined (IS_HILDON)
-
-// 	snprintf (execmdbuf, 512, CONFIG->get_snd_player_args (), sfn.c_str ());
-// 	hildon_play_system_sound (execmdbuf);
-
-// #else  /* POSIX */
-
-// 	snprintf (execmdbuf, 512, CONFIG->get_snd_player_args (), sfn.c_str ());
-
-// 	/**	
-// 	 * Better yet to route WAV playing through esd (if it is running):
-// 	 * esdplay <fname>.wav.
-// 	 *
-// 	 * NOTE: New Gtk 2.4 FileChooserDialog changes disposition mask for SIGCHLD.
-// 	 *       We have to preserve it - otherwise, we would crash and burn.
-// 	 *       COLLECT_STATUS option does that for us here.
-// 	*/
-// 	DL((APP,"POSIX: playing \"%s %s\"\n", 
-// 		CONFIG->get_snd_player_cmd (), execmdbuf));
-
-
-// 	ASSA::Fork::fork_exec (CONFIG->get_snd_player_cmd (), 
-// 						   execmdbuf, ASSA::Fork::IGNORE_STATUS);
-// #endif
-//
-// }
-
 /** 
  *	Deck information consists of two parts: the header (More... dialog)
  *  and a list of cards. Either could have been modified by the user.
@@ -1703,6 +1498,255 @@ test_active_snd_path ()
 	m_snd_enabled = true;
 }
 
+/** Rules for playable name are as follows:
+ *
+ *  <ol>
+ *   <li> A word can be preceeded by "to " to indicate that it is a verb. </li>
+ *
+ *   <li> All multiword expressions are ignored. </li>
+ *
+ *	 <li> The WAV file is assumed to be stored in the directory
+ *     with the name of its first letter, i.e. "granule" will be
+ *     stored in the directory "/path/to/dictionary/g/", and thus 
+ *     the full name would be "/path/to/dictionary/g/granule.wav".
+ *   </li>
+ *
+ *	@return Playable name on success; an empty string on error.
+ */
+void
+Deck::
+play_card (SideSelection side_, Deck::cardlist_iterator& iter_)
+{
+	trace_with_mask("Deck::play_card",GUITRACE|DECK);
+
+	const char* cmd = NULL;
+	static char execmdbuf [PATH_MAX];
+
+	std::vector<string> v;
+
+	string sound_fname;
+	string cmd_args;
+	string file_ext;
+	string w;
+
+	if (!m_snd_enabled) {
+		DL ((GRAPP,"Sound is disabled!\n"));
+		return;
+	}
+
+	if (Deck::s_pipe_stream != NULL) {
+		DL ((GRAPP, "Audio playback is already in progress!\n"));
+		return;
+	}
+
+	DL ((GRAPP,"Playing card audio for %s side.\n", 
+		 (side_ == FRONT ? "Front" : "Back")));
+
+	w = Granule::locale_from_utf8 (
+		(*iter_)->get_alt_spelling (side_ == FRONT ? ASF : ASB));
+
+	if (w.length () == 0) 
+	{
+		w = Granule::locale_from_utf8 ((*iter_)->get_question ());
+		if (w.length () == 0) {
+			DL ((GRAPP,"Conversion for FRONT failed (say, UTF-8 to ASCII).\n"));
+			return;
+		}
+
+		ASSA::Utils::split (w.c_str (), v);
+		if (v.size () > 1) 
+		{
+			if (v[0] != "to") {
+				DL ((GRAPP,"Hmm, v[0] != \"to\"!\n"));
+				return;
+			}
+			w = v[1];
+		}
+	}
+	
+	cmd = CONFIG->get_snd_player_cmd ();
+
+	sound_fname  = m_active_snd_path + G_DIR_SEPARATOR_S;
+	sound_fname += w[0];
+	sound_fname += G_DIR_SEPARATOR_S + w; 
+	
+	cmd_args = CONFIG->get_snd_player_args ();
+
+/** @todo
+ *   This would not work any more because the type of the file 
+ *   (and thus the whole name) is specified in the command line argument.
+ *   But, without this check, what follows might cause a core dump.
+ *   So, you need to fix it somehow!
+ *
+ *	struct stat st;
+ *	if (::stat (sound_fname.c_str (), &st) != 0 || !S_ISREG (st.st_mode)) {
+ *		DL((APP,"Can't stat sound file \"%s\"\n", sound_fname.c_str ()));
+ *		return;
+ *	}
+ */
+	DL((APP,"snd filename=\"%s\"\n", sound_fname.c_str ()));
+
+	/** Arguments composition common to all.
+	 */
+	snprintf (execmdbuf, PATH_MAX, cmd_args.c_str (), sound_fname.c_str ());
+
+    /**======================================================================**
+	 * Win32
+	 * -----
+	 * If sound player command is an empty string, then use
+	 * build-in system PlaySound() 
+	 *   (FIXME: test it if it really works. Looks like you still need
+	 *           the command arguments format to add the sound extension)
+	 *      
+	 * Otherwise, use the external command, which happens by default to be
+	 * granule-audio-player which plays all popular formats regardless.
+	 *
+	 **======================================================================**/
+#if defined (IS_WIN32)
+	int n; 
+ 	if (cmd != NULL && strlen (cmd) != 0) 
+ 	{
+		n = snprintf (execmdbuf, PATH_MAX, "\"%s\" ", cmd);
+
+		/** Surrond command argument by double-quotes - otherwise
+		 *  win32 would garble file path.
+		 */
+		sound_fname = "\"" + sound_fname + "\"";
+		snprintf (execmdbuf+n, PATH_MAX-n, 
+				  cmd_args.c_str(), sound_fname.c_str());
+
+		DL((GRAPP,"WIN32: system(\"%s\")\n", execmdbuf));
+		g_spawn_command_line_async (execmdbuf, NULL);
+	}
+	else {
+		DL((GRAPP,"WIN32: PlaySound(\"%s\")\n", execmdbuf));
+		PlaySound (execmdbuf, NULL, SND_FILENAME| SND_ASYNC);
+	}
+
+    /**======================================================================**
+	 * Hildon
+	 * ------
+	 * We use DPS chip with no impact on main CPU to play mp3 files 
+	 * with GStreamer command-line tool (you need to install 
+	 *
+	 *   gst-launch filesrc location=/path/to/your.mp3 ! dspmp3sink
+	 *
+	 * 
+	 * The expected output from gst-launch is:
+	 *
+	 *   Setting pipeline to PAUSED ...
+	 *   Pipeline is PRELOADED ...
+	 *   Setting pipeline to PLAYING ...
+	 *   New clock: DSPClock
+	 *   Got EOS from element "pipeline0".
+	 *   Execution ended after 56467864151 ns.
+	 *   Setting pipeline to PAUSED ...
+	 *   Setting pipeline to READY ...
+	 *   Setting pipeline to NULL ...
+	 *   FREEING pipeline ...
+	 *
+	 **======================================================================**
+	 * NOTE: New Gtk 2.4 FileChooserDialog changes disposition mask for SIGCHLD.
+	 *       We have to preserve it - otherwise, we would crash and burn.
+	 *       COLLECT_STATUS option does that for us here.
+	 **======================================================================**/
+// #elif defined (IS_HILDON)
+// 	if (cmd != NULL && strlen (cmd) != 0) 
+// 	{
+//  		DL((APP,"HILDON: fork_exec(\"%s %s\")\n", cmd, execmdbuf));
+//  		n = ASSA::Fork::fork_exec (cmd, execmdbuf, ASSA::Fork::COLLECT_STATUS);
+//  		DL((GRAPP,"%s on playing audio clip.\n",
+// 			(n == 0 ? "Success" : (n==1 ? "Failed" : "Failed with wait(2)"))));
+// 	}
+
+	/**======================================================================**
+	 * POSIX (IS_DESKTOP)                                                     *
+	 * ------------------                                                     *
+	 * For now, we squarely rely on external command.                         *
+	 *                                                                        *
+	 * @todo For the next release,  evaluate GStreamer framework.             *
+	 *                                                                        *
+	 **======================================================================**/
+#else
+	if (cmd != NULL && strlen (cmd) != 0) 
+	{
+		if (! GRANULE->async_adapter_enabled ()) {
+			GRANULE->set_async_events_adapter ();
+		}
+
+		w = cmd + string (" ") + execmdbuf;
+		ASSA::AutoPtr<ASSA::Pipe> pipe (new ASSA::Pipe);
+
+		DL ((GRAPP,"POSIX: exec pipe(\"%s\")\n", w.c_str ()));
+
+		Deck::s_pipe_stream = pipe->open (w.c_str (), "r");
+		if (Deck::s_pipe_stream == NULL) {
+			DL ((GRAPP,"POSIX: Pipe::open(\"%s\") failed!\n", w.c_str ()));
+		}
+		else {
+			int val;
+			int fd = pipe->fd ();
+			val = ::fcntl (fd, F_GETFL, 0);
+			val |= O_NONBLOCK;
+			::fcntl (fd, F_SETFL, val);
+			REACTOR->registerIOHandler (this, fd, ASSA::READ_EVENT);
+			m_audio_playback_pipe = pipe.release ();
+			
+		}
+	}
+#endif
+}
+
+/** Drain the pipe stream.
+ */
+int
+Deck::
+handle_read (int fd_)
+{
+	trace_with_mask("Deck::handle_read",GUITRACE);
+
+	int ret = 0;
+	static char buf [1024];
+
+	for (;;) {
+		if ((ret = ::read (fd_, buf, 1024)) < 0) {
+			if (errno == EAGAIN) {
+				break;
+			}
+			else {
+				DL ((GRAPP,"read() error from audio playback pipe : %s\n",
+					 strerror (errno)));
+				return -1;
+			}
+		}
+		
+		else if (ret > 0) { 
+			buf [ret] = '\0';
+			DL ((GRAPP,"GStreamer: %s\n", buf));
+		}
+		else { /* == 0 end-of-stream */
+			return -1;
+		}
+	}
+	
+	return 0;
+}
+
+int
+Deck::
+handle_close (int fd_)
+{
+	trace_with_mask("Deck::handle_close",GUITRACE);
+
+	if (m_audio_playback_pipe != NULL) {
+		delete m_audio_playback_pipe;
+		m_audio_playback_pipe = NULL;
+		Deck::s_pipe_stream = NULL;
+	}
+
+	return 0;
+}
+
 //------------------------------------------------------------------------------
 // LEFTOVERS FROM HEAVY DEBUGGING OF THE PAST
 //------------------------------------------------------------------------------
