@@ -2,7 +2,7 @@
 //------------------------------------------------------------------------------
 //                              Deck.h
 //------------------------------------------------------------------------------
-// $Id: Deck.h,v 1.43 2007/12/02 00:31:24 vlg Exp $
+// $Id: Deck.h,v 1.45 2008/08/04 02:54:31 vlg Exp $
 //
 //  Copyright (c) Vladislav Grinchenko
 //
@@ -21,6 +21,9 @@
 #include "FontsDB.h"
 #include "TextColorsDB.h"
 
+#include <assa/Pipe.h>
+#include <assa/EventHandler.h>
+
 #include <libxml/tree.h>		// xmlDocPtr
 
 typedef enum _ParserState 
@@ -37,7 +40,7 @@ class Card;
  *  Class Deck
  *------------------------------------------------------------------------------
  */
-class Deck : public VDeck
+class Deck : public VDeck, public ASSA::EventHandler
 {
 public:
 	Deck (DeckEditStatus status_);
@@ -45,9 +48,20 @@ public:
 	~Deck ();
 
 	virtual int  load (const string& fname_, DeckAction = INSTALL_DECK);
+
+	/** Handle and discard input from external 
+	 *  audio playback command.
+	 */
+	virtual int handle_read  (int fd_);
+
+	/** Handle audio playback pipe close event.
+	 */
+    virtual int handle_close (int fd_);
+
 	void         insert (const string& from_file_);
 	bool         import_from_csv (const string& file_);
 	bool         export_to_csv   (const string& file_);
+
 	virtual int  save ();
 	virtual int  save_as (const string& fname_);
 
@@ -76,24 +90,28 @@ public:
 	VCard*         find_card_by_id (long id_);
 	DeckEditStatus get_status ();
 
-	const Glib::ustring& get_author    () const { return m_author;        }
-	const Glib::ustring& get_desc      () const { return m_desc;          }
+	const Glib::ustring& get_author () const { return m_author; }
+	const Glib::ustring& get_desc   () const { return m_desc;   }
 
 	void set_author (const Glib::ustring& s_) { m_author = s_; }
 	void set_desc   (const Glib::ustring& s_) { m_desc = s_; }
 
-	const string& get_snd_path  () const { return m_snd_path;      }
-	PathMode      snd_path_mode () const { return m_snd_path_mode; }
+	const string& get_snd_path   () const { return m_snd_path;       }
+	PathMode      snd_path_mode  () const { return m_snd_path_mode;  }
 	PathMode      pics_path_mode () const { return m_pics_path_mode; }
 
-	void set_snd_path  (const string& spath_);
-	void snd_path_mode (PathMode mode_) { m_snd_path_mode = mode_; }
+	void set_snd_path   (const string& spath_);
+	void snd_path_mode  (PathMode mode_) { m_snd_path_mode = mode_; }
 	void pics_path_mode (PathMode mode_) { m_pics_path_mode = mode_; }
 
 	/** Test to see if active_snd_path is a valid path.
 	 */
 	void test_active_snd_path ();
 
+	/** Is audio playback active?
+	 */
+	static bool audio_playback_active () { return s_pipe_stream != NULL; }
+
 	/** Utility functions
 	 */
 	void mark_as_modified     ()       { m_status = MODIFIED_DECK; }
@@ -105,16 +123,26 @@ public:
 	virtual bool with_custom_appearance() const;
 
 	TextColorsDB& text_colors_db () { return m_text_colors_db; }
-	FontsDB&      fonts_db () { return m_fonts_db;   }
-	AppearanceDB&   app_db   () { return m_appearance; }
+	FontsDB&      fonts_db       () { return m_fonts_db;   }
+	AppearanceDB& app_db         () { return m_appearance; }
 
 	void dump () const;
 	void dump_deck_status () const;
 
 private:
+	/** An audio playback pipe. We listen on it to detect
+	 *  the end of the playback.
+	 */
+	static FILE* s_pipe_stream;
+
+	ASSA::Pipe* m_audio_playback_pipe;
+
+private:
+	void init_internals ();
+
 	/** Parse the Deck file card by card.
-		@return 0 on success; -1 on error with error_ explaining the reason.
-	*/
+	 *  @return 0 on success; -1 on error with error_ explaining the reason.
+	 */
 	int  parse_xml_tree (xmlDocPtr parser_, std::string& error_);
 
 	void reset_snd_setup ();
