@@ -1,10 +1,10 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: DeckInfo.cpp,v 1.33 2008/02/04 04:02:20 vlg Exp $
+// $Id: DeckInfo.cpp,v 1.37 2008/08/19 02:52:46 vlg Exp $
 //------------------------------------------------------------------------------
 //                            DeckInfo.cpp
 //------------------------------------------------------------------------------
-//  Copyright (c) 2004 by Vladislav Grinchenko 
+//  Copyright (c) Vladislav Grinchenko 
 //
 //  This program is free software; you can redistribute it and/or 
 //  modify it under the terms of the GNU General Public License   
@@ -91,8 +91,8 @@ DeckInfo (Gtk::Window& parent_, Deck& deck_)
 	m_author_entry   (Gtk::manage (new Gtk::Entry ())),
 	m_desc_entry     (Gtk::manage (new Gtk::Entry ())),
 	m_snd_path_entry (Gtk::manage (new Gtk::Entry ())),
-	m_appearance_changed (false),
-	m_response (Gtk::RESPONSE_CANCEL)
+	m_response (Gtk::RESPONSE_CANCEL),
+	m_appearance_changed (false)
 {  
 	trace_with_mask("DeckInfo::DeckInfo", GUITRACE);
 
@@ -239,31 +239,36 @@ DeckInfo (Gtk::Window& parent_, Deck& deck_)
 	TextAlignmentsUI::value_list_t align_x_list;
 	TextAlignmentsUI::value_list_t align_y_list;
 	TextAlignmentsUI::value_list_t paragraph_list;
+	TextAlignmentsUI::value_list_t img_position_list;
 
-	align_x_list   = TextAlignmentsUI::make_align_x_list ();
-	align_y_list   = TextAlignmentsUI::make_align_y_list ();
-	paragraph_list = TextAlignmentsUI::make_paragraph_list ();
+	align_x_list      = TextAlignmentsUI::make_align_x_list      ();
+	align_y_list      = TextAlignmentsUI::make_align_y_list      ();
+	paragraph_list    = TextAlignmentsUI::make_paragraph_list    ();
+	img_position_list = TextAlignmentsUI::make_img_position_list ();
 
 	m_front_widget=new TextAlignmentsUI("<b>Front Text Alignment</b>",
-											  align_x_list,
-											  align_y_list,
-											  paragraph_list);
+										align_x_list,
+										align_y_list,
+										paragraph_list,
+										img_position_list);
 
 	m_back_widget=new TextAlignmentsUI("<b>Back Text Alignment</b>",
-											 align_x_list,
-											 align_y_list,
-											 paragraph_list);
+									   align_x_list,
+									   align_y_list,
+									   paragraph_list,
+									   img_position_list, false);
 
 	m_example_widget=new TextAlignmentsUI("<b>Example Text Alignment</b>",
-												align_x_list,
-												align_y_list,
-												paragraph_list);
+										  align_x_list,
+										  align_y_list,
+										  paragraph_list,
+										  img_position_list);
 
 	/** Load data from the model
 	 */
 	load_appearance ();
 
-	m_inner_app_vbox->pack_start (*m_fonts_widget, Gtk::PACK_SHRINK, 4);
+	m_inner_app_vbox->pack_start (*m_fonts_widget,  Gtk::PACK_SHRINK, 4);
 	m_inner_app_vbox->pack_start (*m_colors_widget, Gtk::PACK_SHRINK, 4);
 
 	m_inner_app_vbox->pack_start (*(m_front_widget->frame ()), 
@@ -366,8 +371,9 @@ DeckInfo (Gtk::Window& parent_, Deck& deck_)
 	DI_SET_FRAME(snd_frame, snd_vbox);
 
 	/********************
-	 * Setup Pic Path
-	 ********************/
+	 * Setup Pic Path   *
+	 ********************
+	 */
 	if (m_deck.pics_path_mode () == PATH_RELATIVE) {
 		DI_SET_CHECKBUTTON(m_enable_rel_pics_path, true);
 	}
@@ -379,8 +385,9 @@ DeckInfo (Gtk::Window& parent_, Deck& deck_)
 	DI_SET_FRAME(pic_frame, pic_vbox);
 
 	/********************
-	 * Setup Appearance
-	 ********************/
+	 * Setup Appearance *
+	 ********************
+	 */
 	bool app_enabled = m_deck.with_custom_appearance ();
 	DI_SET_CHECKBUTTON(m_enable_custom_app, app_enabled);
 	m_inner_app_vbox->set_sensitive (app_enabled);
@@ -391,8 +398,9 @@ DeckInfo (Gtk::Window& parent_, Deck& deck_)
 	DI_SET_FRAME(app_frame, app_vbox);
 
 	/********************
-	 * Pack all up
-	 ********************/
+	 * Pack all up      *
+	 ********************
+	 */
 	Gtk::VBox* vbox_all = Gtk::manage (new Gtk::VBox (false, 0));
 
 	vbox_all->pack_start (*info_table, Gtk::PACK_SHRINK, 4);
@@ -421,8 +429,12 @@ DeckInfo (Gtk::Window& parent_, Deck& deck_)
 	get_action_area ()->pack_start (*m_apply_button);
 
 	/************************
-	 * Initialize callbacks 
-	 ************************/
+	 * Initialize callbacks *
+	 ************************
+	 */
+ 	signal_delete_event ().connect (
+ 		sigc::mem_fun (*this, &DeckInfo::on_delete_window_clicked));
+
 	m_cancel_button->signal_clicked ().connect (
 		sigc::mem_fun (*this, &DeckInfo::on_cancel_clicked));
 
@@ -484,19 +496,25 @@ on_enable_rel_snd_path_clicked ()
 {
 	trace_with_mask("DeckInfo::on_enable_rel_snd_path_clicked", GUITRACE);
 
+	DL ((GRAPP,"1\n"));
 	Glib::ustring path_entry = m_snd_path_entry->get_text ();
+
+	DL ((GRAPP,"2\n"));
 	bool enabled = m_enable_rel_snd_path->get_active ();
 
 	if (enabled) {
+		DL ((GRAPP,"e-1\n"));
 		string relpath = Granule::calculate_relative_path (
 			m_deck.get_deck_path (), 
 			Granule::filename_from_utf8 (m_snd_path_entry->get_text ()));
 
+		DL ((GRAPP,"e-2\n"));
         m_snd_path_entry->set_text (relpath);
 		DL ((GRAPP,"Enable relative snd path '%s'\n", relpath.c_str ()));
 	}
 	else {
 		if (!path_entry.empty ()) {
+			DL ((GRAPP,"noob-1\n"));
 			string abspath = Granule::calculate_absolute_path (
 				m_deck.get_deck_path (), path_entry);
 			DL ((GRAPP,"Disable relative snd path.\n"));
@@ -602,6 +620,7 @@ on_apply_clicked ()
 
 	active = m_enable_rel_pics_path->get_active ();
 	pmode = active ? PATH_RELATIVE : PATH_ABSOLUTE;
+
 	if (m_deck.pics_path_mode () != pmode) {
 		m_deck.pics_path_mode (pmode);
 		m_deck.mark_as_modified ();
@@ -614,14 +633,28 @@ on_apply_clicked ()
 		m_deck.set_custom_appearance ();
 		m_deck.mark_as_modified ();
 	}
-	save_appearance ();
 
+	save_appearance ();
 	m_appearance_changed = true;
 
 	m_response = Gtk::RESPONSE_APPLY;
+	hide ();
 	gtk_main_quit ();
 }
 
+/**
+ * @return true - top other handlers from being invoked for the event.
+ */
+bool
+DeckInfo::
+on_delete_window_clicked (GdkEventAny* /*event_*/)
+{
+    trace_with_mask("DeckInfo::on_delete_window_clicked",GUITRACE);
+
+	on_cancel_clicked ();
+    return true;  
+}
+
 void
 DeckInfo::
 on_cancel_clicked ()
@@ -629,9 +662,9 @@ on_cancel_clicked ()
 	trace_with_mask("DeckInfo::on_cancel_clicked", GUITRACE);
 
 	m_response = Gtk::RESPONSE_CANCEL;
+	hide ();
 	gtk_main_quit ();
 }
-	
 
 /**-----------------------------------------------------------------------------
  ** Alignment widgets.
@@ -668,7 +701,8 @@ run ()
 	trace_with_mask("DeckInfo::run", GUITRACE);	
 
 	show ();
-	Gtk::Main::run ();
+	present ();
+	gtk_main ();
 
 	return m_response;
 }
