@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: DeckList.cpp,v 1.21 2006/12/11 03:07:22 vlg Exp $
+// $Id: DeckList.cpp,v 1.27 2008/02/04 04:02:20 vlg Exp $
 //------------------------------------------------------------------------------
 //                            DeckList.cpp
 //------------------------------------------------------------------------------
@@ -26,6 +26,8 @@
 #include "Granule.h"
 #include "MainWindow.h"
 
+using sigc::mem_fun;
+
 //------------------------------------------------------------------------------
 // Static functions
 //
@@ -57,13 +59,15 @@ DeckList () : m_selected_deck (NULL)
 	set_policy (Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
 
 	m_list_store_ref = Gtk::ListStore::create (m_columns);
+
 	m_tree_view.set_model (m_list_store_ref);
 	m_tree_view.set_rules_hint ();
+	m_tree_view.set_headers_clickable (false);
+	m_tree_view.set_headers_visible (true);
 
 	int cnt = m_tree_view.append_column ("Decks", m_columns.m_name);
 	Gtk::TreeViewColumn* column = m_tree_view.get_column (cnt - 1);
 	column->set_sizing (Gtk::TREE_VIEW_COLUMN_FIXED);
-//	column->set_fixed_width (60);
 	column->set_resizable ();
 	
 	/** Enable column sorting
@@ -74,16 +78,19 @@ DeckList () : m_selected_deck (NULL)
 	m_tree_sel_ref = m_tree_view.get_selection ();
 	m_tree_sel_ref->set_mode (Gtk::SELECTION_SINGLE);
 	m_tree_sel_ref->signal_changed ().connect (
-		sigc::mem_fun (*this, &DeckList::on_name_selected));
+		mem_fun (*this, &DeckList::on_name_selected));
 
 	m_tree_view.add_events (Gdk::BUTTON_PRESS_MASK);
 
 	m_tree_view.signal_button_press_event ().connect (
-		sigc::mem_fun (*this, &DeckList::on_mouse_click), false);
+		mem_fun (*this, &DeckList::on_mouse_click), false);
+
+	m_tree_view.signal_key_press_event ().connect (
+		mem_fun (*this, &DeckList::on_key_tapped), false);
 
 #ifdef IS_HILDON
 	m_tree_view.signal_row_activated ().connect (
-		sigc::mem_fun (*this, &DeckList::on_row_activated), false);
+		mem_fun (*this, &DeckList::on_row_activated), false);
 #endif
 
 	/** Enable DeckList rows as DND source. Target entry of the source
@@ -95,10 +102,12 @@ DeckList () : m_selected_deck (NULL)
 	list_targets.push_back (Gtk::TargetEntry ("text/plain"));
 
 	m_tree_view.enable_model_drag_source (list_targets);
+
 	m_tree_view.signal_drag_data_get ().connect (
-		sigc::mem_fun (*this, &DeckList::on_drag_data_get));
+		mem_fun (*this, &DeckList::on_drag_data_get));
+
 	m_tree_view.signal_drag_begin ().connect (
-		sigc::mem_fun (*this, &DeckList::on_drag_begin));
+		mem_fun (*this, &DeckList::on_drag_begin));
 
 	/** Pack it up.
 	 */
@@ -143,7 +152,7 @@ init_popup_menu ()
 	Menu* add2box_menu = manage (new Menu ());
 	MenuList& a2bp = add2box_menu->items ();
 
-	for (int j = 0; j < CARD_BOX_SIZE; j++) {
+	for (int j = 0; j < CARDBOXES_MIN; j++) {
 		std::ostringstream os;
 		os << "Card box " << (j + 1);
 		a2bp.push_back (MenuElem (os.str (), 
@@ -202,7 +211,7 @@ on_name_selected (void)
 		TOPMENUBAR->set_deck_sensitivity (true, size ());
 	}
 	else {
-		DL ((GRAPP,"None of the rows is selected!\n"));
+		DL ((GRAPP,"Row selection is empty!\n"));
 	}
 }
 
@@ -303,31 +312,37 @@ on_drag_data_get (const Glib::RefPtr<Gdk::DragContext>& context_,
 	MAINWIN->set_dnd_source (m_selected_deck);
 }
 
-/** Called when button is pressed on any of the rows.
-	We filter out Right-Click to bring up a popup dialog or
-	double-click to bring DeckPlayer window.
-
-	The rest of the events are handled by the default event
-	handler of TreeView afterwards.
+/** 
+ * This method is called when button is pressed on any of the rows.
+ *
+ * We filter out Right-Click to bring up a popup dialog or
+ * double-click to bring DeckPlayer window.
+ *
+ * The rest of the events are handled by the default event
+ * handler of TreeView afterwards.
 */
 bool
 DeckList::
 on_mouse_click (GdkEventButton* event_)
 {
 	trace_with_mask("DeckList::on_mouse_click", GUITRACE);
+	bool ret = false;
 
-	if (event_->type == GDK_2BUTTON_PRESS
+	if (event_->type == GDK_2BUTTON_PRESS 
 #ifdef IS_HILDON
 		|| event_->type == GDK_Escape // Hildon round button (Cancel,Close)
 #endif
 		) 
 	{
 		DECKMGR->play_deck_cb ();
-		return false;			// returning 'true' results in DND behavior !?!
 	}
-	else if (event_->type == GDK_BUTTON_PRESS && event_->button == 3) {
+	else if (event_->type == GDK_BUTTON_PRESS && event_->button == 3) 
+	{
 		m_popup_menu.popup (event_->button, event_->time);
-		return true;
+		/** 
+		 * Returning 'true' results in DND behavior !?!
+		 */
+		ret = true;
 	}
 
 #ifdef IS_HILDON
@@ -336,14 +351,49 @@ on_mouse_click (GdkEventButton* event_)
 	on_name_selected ();
 #endif
 
-	return false;
+	/* Emmit our [clicked] signal
+	 */
+	m_signal_mouse_clicked.emit ();
+
+	return ret;
 }
 
-#ifdef IS_HILDON
+DeckList::signal_mouse_clicked_type 
+DeckList::
+signal_mouse_clicked ()
+{
+	trace_with_mask("DeckList::signal_mouse_clicked", GUITRACE);
+	return m_signal_mouse_clicked;
+}
+
+/**
+ * Handler [Enter] key.
+ *
+ */
+bool
+DeckList::
+on_key_tapped (GdkEventKey* event_)
+{
+	trace_with_mask("DeckList::on_key_tapped", GUITRACE);
+
+	bool ret = false;
+
+	DL ((GRAPP,"Key event: 0x%x\n", event_->type));
+	DL ((GRAPP,"Key pressed: 0x%x\n", event_->keyval));
+
+	if (event_->type == GDK_KEY_PRESS && event_->keyval == GDK_Return)
+	{
+		DECKMGR->play_deck_cb ();
+		ret = true;
+	}
+		
+	return ret;
+}
 
 /** 
  * Catch Select (round button) press event with a selected row
  */
+#ifdef IS_HILDON
 void
 DeckList::
 on_row_activated (const Gtk::TreePath& path_, Gtk::TreeViewColumn* column_)
@@ -352,4 +402,4 @@ on_row_activated (const Gtk::TreePath& path_, Gtk::TreeViewColumn* column_)
 
 	DECKMGR->play_deck_cb ();
 }
-#endif
+#endif	// IS_HILDON
