@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: DeckList.h,v 1.10 2006/03/19 15:39:44 vlg Exp $
+// $Id: DeckList.h,v 1.12 2008/01/14 03:41:00 vlg Exp $
 //------------------------------------------------------------------------------
 //                            DeckList.h
 //------------------------------------------------------------------------------
@@ -28,6 +28,9 @@
 
 class DeckList : public Gtk::ScrolledWindow
 {
+public:
+	typedef sigc::signal<void> signal_mouse_clicked_type;
+
 public:
 	DeckList ();
 	void init_popup_menu ();
@@ -56,6 +59,10 @@ public:
 	/*< callbacks >*/
 	void on_name_selected (void);
 	bool on_mouse_click (GdkEventButton* event_);
+	bool on_key_tapped (GdkEventKey* event_);
+
+	/*< Singals >*/
+	signal_mouse_clicked_type signal_mouse_clicked ();
 
 	/** Callback called when user starts dragging Deck item towards
 		a CardDeck icon.
@@ -104,6 +111,8 @@ private:
 
 	/// Popup menu
 	Gtk::Menu m_popup_menu;
+
+	signal_mouse_clicked_type m_signal_mouse_clicked;
 };
 
 inline
