@@ -1,10 +1,10 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: DeckPlayer.h,v 1.74 2008/02/04 04:02:20 vlg Exp $
+// $Id: DeckPlayer.h,v 1.88 2008/08/16 03:13:09 vlg Exp $
 //------------------------------------------------------------------------------
 //                            DeckPlayer.h
 //------------------------------------------------------------------------------
-//  Copyright (c) 2004-2007 Vladislav Grinchenko 
+//  Copyright (c) Vladislav Grinchenko 
 //
 //  This program is free software; you can redistribute it and/or 
 //  modify it under the terms of the GNU General Public License   
@@ -30,6 +30,7 @@
 #include <gtkmm/textview.h>
 #include <gtkmm/uimanager.h>
 #include <gtkmm/table.h>
+#include <gtkmm/toggletoolbutton.h>
 
 #ifdef IS_HILDON
 #include <hildonmm.h>
@@ -38,6 +39,8 @@
 #include "GrappConf.h"
 #include "VDeck.h"
 
+#include <vector>
+
 class Gtk::ProgressBar;
 class Gtk::EventBox;
 
@@ -47,9 +50,11 @@ class TestLineEntry;
 class VerifyControl;
 class ButtonWithImageLabel;
 
-/**=============================================================================
- *   Class SideFlipControl encapsulates the side switch radio buttons
- **=============================================================================
+/**===========================================================================**
+ *                                                                             *
+ *   Class SideFlipControl encapsulates the side switch radio buttons.         *
+ *                                                                             *
+ **===========================================================================**
  */
 class SideFlipControl : public Gtk::EventBox
 {
@@ -62,23 +67,17 @@ private:
 	void on_flip_button_clicked ();
 
 private:
-	DeckPlayer&      m_parent;
+	DeckPlayer& m_parent;
 
-	/** For desktop and Hildon
-	 */
-    Gtk::RadioButton m_radio_fb;
-    Gtk::RadioButton m_radio_bb;
-	Gtk::VButtonBox  m_button_box;
-
-	/** For PDA 
-	 */
 	Gtk::Button      m_flip_button;
 	SideSelection    m_side;
 };
 
-/**=============================================================================
- *                              Class DeckPlayer 
- **=============================================================================
+/**===========================================================================**
+ *                                                                             *
+ *                              Class DeckPlayer                               *
+ *                                                                             *
+ **===========================================================================**
  */
 #ifdef IS_HILDON 
 class DeckPlayer : public Hildon::Window
@@ -96,11 +95,7 @@ public:
 
 	/** We block [x] delete action and reroute it to <Close>
 	 */
-	virtual bool on_delete_event (GdkEventAny* event_) 
-	{
-		on_close_clicked ();
-		return true;
-	}
+	bool on_delete_window_clicked (GdkEventAny* event_);
 
 	/** Called in response to pressing <Close> button
 	 */
@@ -120,9 +115,13 @@ public:
 	*/
 	void size_allocate_cb (Gtk::Allocation& size_);
 
-	/** User pressed <Say It> button to play WAV audio, if available.
+	/** Clear selection on notebook page switch
 	 */
-	void on_sayit_clicked ();
+	void on_notebook_page_flipped (GtkNotebookPage* page_, guint page_num_);
+
+	/** User pressed <Sound> button to play WAV audio (if available).
+	 */
+	void on_play_sound_clicked ();
 
 	/** Shuffle the cards in the deck
 	 */
@@ -140,11 +139,12 @@ public:
 
 	// -*- Button Actions -*-
 	void go_first_cb ();		// Go to the first card
-	void go_prev_cb ();			// Go back one card
-	void go_next_cb ();			// Go to the next card
-	void go_last_cb ();			// Go to the last card
-	void know_answer_cb ();		// I know the answer
-	void no_clue_cb ();			// Have no clue about this one
+	void go_prev_cb  ();		// Go back one card
+	void go_next_cb  ();		// Go to the next card
+	void go_last_cb  ();		// Go to the last card
+
+	// -*- Answer Callbacks -*-
+	void process_answer_cb (AnswerQuality qa_);
 
 	// VerifyControl is enabled and user clicked [Check] button
 	void on_verify_clicked ();
@@ -155,11 +155,14 @@ public:
 	// -*- Catch and process keyboard events -*-
 	bool on_key_pressed (GdkEventKey* evt_);
 
-	// Show/hide verify control hbox (for IS_PDA mode)
-	void on_verify_control_hide_clicked ();
+	// Show/Hide VerifyControl toolbar (sticky).
+	void on_verify_control_checkbutton_clicked ();
 
-	// Show/hide navigations button toolbar (for IS_PDA mode)
-	void on_navigation_toolbar_hide_clicked ();
+	// Show/Hide StatsInfo toolbar (sticky).
+	void on_stats_info_checkbutton_clicked ();
+
+	// Show/Hide Sidebar (sticky).
+	void on_sidebar_checkbutton_clicked ();
 
 	/** Get the deck pointer for comparison
 	 */
@@ -214,8 +217,12 @@ public:
 	/** Extend Stock Icons with application-specific ones.
 	 */
 	static Gtk::StockID SAY_IT;
-	static Gtk::StockID THUMB_UP;
-	static Gtk::StockID THUMB_DOWN;
+	static Gtk::StockID THUMB_0;
+	static Gtk::StockID THUMB_1;
+	static Gtk::StockID THUMB_2;
+	static Gtk::StockID THUMB_3;
+	static Gtk::StockID THUMB_4;
+	static Gtk::StockID THUMB_5;
 
 private:
 	void show_deck       ();	// Show first card of the loaded deck
@@ -224,6 +231,14 @@ private:
 	void set_sensitivity ();    // De-sensetize inappropriate controls
 	void set_win_name    ();    // Set tile name
 	void clear_selection ();    // Clear any selected text (annoying)
+	void set_controls_visibility ();   // If needed, hide optional controls.
+	void save_controls_visibility ();  // Save visibility of controls.
+
+	void enable_good_answer_buttons ();
+	void enable_bad_answer_buttons ();
+
+	void scroll_up ();
+	void scroll_down ();
 
 	void reset_text_fonts ();
 	void reset_text_colors ();
@@ -235,6 +250,12 @@ private:
 	int x_padding_val (SideSelection s_);
 	int y_padding_val (SideSelection s_);
 
+	/** Return image position in relation to text (vertical).
+	 *  @return TA_IMG_POSITION_TOP for image above text;
+	 *          TA_IMG_POSITION_BOTTOM for image below text;
+	 */
+	int image_valignment (SideSelection side_);
+
 	/** If deck is empty, paralize controls. Deck can be loaded empty,
 	 *  or become empty once the last card is removed. The reverse
 	 *  applies as soon as first card is added.
@@ -258,29 +279,56 @@ private:
 	 */
 	bool markup_is_valid (VCard* card_);
 
-	bool check_F6_key (gint keyval_);
-	bool check_F7_key (gint keyval_);
-	bool check_F8_key (gint keyval_);
+	void go_to_fullscreen ();
+	bool check_fullscreen_key (gint keyval_);  // F6 key
+	bool check_F7_key (gint keyval_);          // F7 (hildon zoom in)
+	bool check_F8_key (gint keyval_);          // F8 (hildon zoom out)
 
 	Gtk::VBox* get_vbox () { return m_vbox; }
+	Gtk::HBox* get_enclosure () { return m_enclosure_hbox; }
+
+	/** A utility function similar to one found in EditControls.
+	 */
+	static void 
+	add_pixmap_to_factory (Gtk::StockID& id_,
+						   const Glib::ustring& label_,
+						   Glib::RefPtr<Gdk::Pixbuf> pixbuf_,
+						   Glib::RefPtr<Gtk::IconFactory> ifactory_,
+						   bool large_icons_ = false);
+
+	/** Make a 72x72 image button suitable for sidebar controls
+	 *  display in fullscreen mode with touchscreen platforms (N8x0).
+	 */
+	static Gtk::Button* make_button_with_image (const char* pixmap_path_);
+
+	/** Go full-screen and back to normal size.
+	 */
+	void go_fullscreen_mode ();
+
+	void hide_sidebar () { m_ctrl_sidebar_vbox->hide (); }
+	void show_sidebar () { m_ctrl_sidebar_vbox->show (); }
+	void change_sidebar_location ();
 
 private:
 	static const string RATIO_TITLE;
 	static const string REVIEW_TITLE;
 	static const string RIGHT_WRONG_TITLE;
 
-#ifdef IS_HILDON
-	bool              m_hildon_fullscreen;
-#endif
+private:
+	typedef std::vector<Gtk::Widget*> answers_type;
+	typedef answers_type::iterator answers_iterator;
 
+	ButtonWithImageLabel* m_play_sound_button;	 // Play the sound
 	ButtonWithImageLabel* m_close_button;
-	ButtonWithImageLabel* m_shuffle_button;
-	ButtonWithImageLabel* m_edit_button;         // View/Edit a Deck
-	ButtonWithImageLabel* m_vcard_edit_button;   // Edit a Card
+
+	Gtk::Widget*      m_shuffle_button;      // Shuffle Deck
+	Gtk::Widget*      m_edit_button;	     // View/Edit Deck
+	Gtk::Widget*      m_vcard_edit_button;   // Edit Card
 
 	int               m_match_front_row;     // Which row to select for answer
 
 	Gtk::Label        m_count_label;         // Reviewed/Total
+	Gtk::Label        m_count_label_bare;    // R/T (review/total)
 	Gtk::Label        m_success_ratio_label; // Success ratio
 	Gtk::Label        m_right_wrong_label;   // Right/Wrong answers
 	Gtk::Image*       m_exp_light;           // Expiration light
@@ -320,20 +368,16 @@ private:
 	Gtk::Widget*      m_toolbar_go_prev;
 	Gtk::Widget*      m_toolbar_go_next;
 	Gtk::Widget*      m_toolbar_go_last;
-	Gtk::Widget*      m_toolbar_know_it;
-	Gtk::Widget*      m_toolbar_no_clue;
 
-	Glib::RefPtr<Gdk::Pixbuf> m_playit_xpm;	// Sound icon
-	Gtk::Image*       m_playit_icon;
+	answers_type      m_toolbar_answers_ok;
+	answers_type      m_toolbar_answers_bad;
 
-	/** For compact portrait-like PDA mode
-	 */
-	Glib::RefPtr<Gdk::Pixbuf> m_hide_control;
-	Glib::RefPtr<Gdk::Pixbuf> m_show_control;
-	Gtk::Button*      m_hs_button_vc;
-	Gtk::Button*      m_hs_button_ntb;
+	answers_type      m_sidebar_answers_ok;
+	answers_type      m_sidebar_answers_bad;
 
-	VDeck&            m_deck;
+	Glib::RefPtr<Gdk::Pixbuf> m_play_sound_xpm;	// Sound icon
+
+	VDeck&                   m_deck;
 	VDeck::cardlist_iterator m_deck_iter; // Position in the Deck
 
 	bool m_is_lesson_learned;	// Have user gone through the deck?
@@ -341,50 +385,57 @@ private:
 
 	Glib::RefPtr<Gtk::UIManager>   m_uimanager;
 	Glib::RefPtr<Gtk::ActionGroup> m_actgroup;
+	Glib::RefPtr<Gtk::ActionGroup> m_actgroup_extra;
 
 	bool m_tab_activated;		// Leave Tab+Spacebar/Enter event to Dialog
 	bool m_initialized;	        // Has the object been fully constructed?
 
-	Gtk::VBox*     m_vbox;
+	Gtk::HBox*     m_enclosure_hbox;  // The outer box (sidebar + m_vbox)
+	Gtk::VBox*     m_vbox;            // Inner box (notebook + controls)
+
 	Gtk::HBox*     m_navigation_controls_hbox;
-	Gtk::Toolbar*  m_ctrl_toolbar;
 	Gtk::EventBox* m_nav_ctrl_evbox;
+
+	Gtk::HBox*     m_extra_controls_hbox;
+	Gtk::HBox*     m_stats_info_hbox;
+
+	Gtk::Toolbar*  m_answers_toolbar;  /** A collection of answer buttons 
+										*  0-5 embedded inside bottom toolbar.*/
+	Gtk::Toolbar*  m_bottom_ctrl_toolbar;
+
+ 	Gtk::Toolbar*  m_ctrl_sidebar;     /** A full-screen optional sidebar */
+	Gtk::VBox*     m_ctrl_sidebar_vbox;
+
+	Gtk::ScrolledWindow* m_front_scrollw;
+	Gtk::ScrolledWindow* m_back_scrollw;
+
+    /** Hide/Show VerifyControl toolbar.
+	 */
+	Gtk::ToggleToolButton* m_vc_toolbar_button; 
+
+    /** Hide/Show statistics toolbar.
+	 */
+	Gtk::ToggleToolButton* m_stats_toolbar_button; 
+
+    /** Hide/Show vertical sidebar.
+	 */
+	Gtk::ToggleToolButton* m_sbar_toolbar_button; 
+
+	/** Tracks the state of the fullscreen mode.
+	 */
+	bool m_fullscreen_mode;
+
 };
 
-//------------------------------------------------------------------------------
-// Inline functions
-//------------------------------------------------------------------------------
-inline void
-DeckPlayer::
-clear_text ()
-{
-	m_front.  set_text ("");
-	m_back.   set_text ("");
-	m_example.set_text ("");
-	m_front_image->hide ();
-}
 
-/*------------------------------------------------------------
- * Madness I have to live with just for the sake of nokia 770
- *------------------------------------------------------------
+/**---------------------------------------------------------------------------**
+ * Inline functions                                                            *
+ **---------------------------------------------------------------------------**
  */
 
-inline bool
-DeckPlayer::
-check_F6_key (gint keyval_)
-{
-#ifdef IS_HILDON
-	if (keyval_ == GDK_F6) { // Full screen
-		m_hildon_fullscreen != m_hildon_fullscreen;
-#ifdef HILDON_DEPRICATED
-		set_fullscreen (m_hildon_fullscreen);
-#endif
-		return true;
-	}
-#endif
-	return false;
-}
-
+/**
+ * Madness I have to live with just for the sake of nokia 770
+ */
 inline bool
 DeckPlayer::
 check_F7_key (gint keyval_)
