@@ -1,10 +1,10 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: DeckView.cpp,v 1.66 2008/02/04 04:02:20 vlg Exp $
+// $Id: DeckView.cpp,v 1.71 2008/07/21 03:04:40 vlg Exp $
 //------------------------------------------------------------------------------
 //                            DeckView.cpp
 //------------------------------------------------------------------------------
-//  Copyright (c) 2004,2006 by Vladislav Grinchenko 
+//  Copyright (c) Vladislav Grinchenko 
 //
 //  This program is free software; you can redistribute it and/or 
 //  modify it under the terms of the GNU General Public License   
@@ -55,9 +55,6 @@ using sigc::mem_fun;
 DeckView::
 DeckView (VDeck& deck_, int selected_row_) 
 		:
-#ifdef IS_HILDON
-//	Hildon::Window    ("Deck View"),
-#endif
 	m_label_count      (manage (new Gtk::Label ("0 cards"))),
 	m_deck             (deck_),
 	m_row_idx          (selected_row_),
@@ -105,6 +102,9 @@ DeckView (VDeck& deck_, int selected_row_)
 	m_close_button->signal_clicked ().connect (
 		mem_fun (*this, &DeckView::on_close_clicked));
 
+ 	signal_delete_event ().connect (
+ 		sigc::mem_fun (*this, &DeckView::on_delete_window_clicked));
+
 	/** 
 	 * For the top edit buttons bar, create an ActionGroup.
 	 */
@@ -129,7 +129,7 @@ DeckView (VDeck& deck_, int selected_row_)
 					 mem_fun (*this, &DeckView::on_autofill_clicked));
 
 	m_actgroup->add (Gtk::Action::create ("Delete", 
-#ifdef WIN32
+#ifdef IS_WIN32
 										  Gtk::Stock::CUT,
 #else
 										  Gtk::Stock::DELETE,
@@ -466,7 +466,11 @@ run (Gtk::Window& parent_)
     trace_with_mask("DeckView::run",GUITRACE);
 
 	GRANULE->move_child_to_parent_position (parent_, *this);
+
+	show ();
+	present ();
 	gtk_main ();
+
 	GRANULE->move_parent_to_child_position (parent_, *this);
 
 	return (Gtk::RESPONSE_NONE);
@@ -478,6 +482,7 @@ on_close_clicked ()
 {
 	trace_with_mask("DeckView::on_close_clicked", GUITRACE);
 
+	hide ();
 	gtk_main_quit ();
 }
 
@@ -531,8 +536,10 @@ add_new_card (Card* card_)
 
 	m_tree_sel_ref->unselect_all ();
 	m_tree_sel_ref->select (row);
+
 	set_cards_count ();
 	autoscroll_down ();
+
 	m_modified = true;
 }
 
@@ -542,9 +549,8 @@ on_add_clicked ()
 {
 	trace_with_mask("DeckView::on_add_clicked", GUITRACE);
 
-	/** It is safe to cast VDeck& to Deck& here
-	 *  because we can only perform add/edit operation on
-	 *  a real Deck.
+	/** It is safe to cast VDeck& to Deck& here because we can only 
+	 *  perform add/edit operation with a real Deck.
 	 */
 	CardView cview (new Card (dynamic_cast<Deck&>(m_deck)));
 
@@ -925,10 +931,11 @@ on_add_card_to_cardbox ()
 }
 
 /**
-   Sort Front column of words.
-   The parts of the text may be marked up with Pango markup language.
-   Remove markup. Also, remove 'to ', 'a ', 'an ' prefixes.
-   Then compare the first word.
+ * Sort Front column of words.
+ *
+ * The parts of the text may be marked up with Pango markup language.
+ * Remove markup. Also, remove 'to ', 'a ', 'an ' prefixes.
+ * Then compare the first word.
 */
 int 
 DeckView::
@@ -938,17 +945,17 @@ sort_compare_impl (const Gtk::TreeModel::iterator& a_,
 {
     trace_with_mask("DeckView::sort_compare_impl",GUITRACE);
 
-	gchar* txt_a;
-	gchar* txt_b;
-	gchar* pa;
-	gchar* pb;
-	int    ret;
+	Glib::ustring a;
+	Glib::ustring b;
 
 	const Gtk::TreeModel::Row row_a = *a_;
 	const Gtk::TreeModel::Row row_b = *b_;
-	
-	Glib::ustring a;
-	Glib::ustring b;
+
+	gchar* txt_a;
+	gchar* txt_b;
+	int    ret = 0;
+
+	txt_a = txt_b = NULL;
 
 //	DL ((GRAPP, "Sorting %s column\n", (column_ == 0 ? "Front" : "Back")));
 
@@ -961,21 +968,37 @@ sort_compare_impl (const Gtk::TreeModel::iterator& a_,
 		b = row_b [m_columns.m_back_row];
 	}
 
-	pa = txt_a = Granule::strip_pango_markup (a.c_str ());
-	pb = txt_b = Granule::strip_pango_markup (b.c_str ());
+	/** 
+	 * We need a hold on the pointers to deallocate memory.
+	 */
+	txt_a = Granule::strip_pango_markup (a.c_str ());
+	txt_b = Granule::strip_pango_markup (b.c_str ());
 
 	Granule::remove_common_prefixes (txt_a);
 	Granule::remove_common_prefixes (txt_b);
 
-//	DL ((APP,"Compare: \"%s\" vs. \"%s\"\n", txt_a, txt_b));
-
-	ret = strcmp (txt_a, txt_b);
-	g_free (pa);
-	g_free (pb);
+	if (txt_a != NULL || txt_b != NULL) {
+		ret = strcmp (txt_a, txt_b);
+		g_free (txt_a);
+		g_free (txt_b);
+	}
 
 	return ret;
 }
 
+/**
+ * @return true - top other handlers from being invoked for the event.
+ */
+bool
+DeckView::
+on_delete_window_clicked (GdkEventAny* /*event_*/)
+{
+    trace_with_mask("DeckView::on_delete_window_clicked",GUITRACE);
+
+    on_close_clicked ();
+    return true;  
+}
+
 //==============================================================================
 //                              Utilities
 //------------------------------------------------------------------------------
@@ -1095,3 +1118,4 @@ on_key_tapped (GdkEventKey* event_)
 
 	return ret;
 }
+
