@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: DeckView.h,v 1.40 2008/02/04 04:02:21 vlg Exp $
+// $Id: DeckView.h,v 1.41 2008/06/02 03:03:54 vlg Exp $
 //------------------------------------------------------------------------------
 //                            DeckView.h
 //------------------------------------------------------------------------------
@@ -62,11 +62,7 @@ public:
 
 	/** We block [x] delete action and reroute it to <Close>
 	 */
-	virtual bool on_delete_event (GdkEventAny* event_) 
-	{
-		on_close_clicked ();
-		return true;
-	}
+	bool on_delete_window_clicked (GdkEventAny* event_);
 
 	/** Has the Deck been modified so that it might invalidate
 	 *	the iterators? Deleting a card and moving card(s) up/down
