@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: EditControls.cpp,v 1.13 2008/01/11 03:52:46 vlg Exp $
+// $Id: EditControls.cpp,v 1.14 2008/05/24 22:58:51 vlg Exp $
 //------------------------------------------------------------------------------
 //                            EditControls.cpp
 //------------------------------------------------------------------------------
@@ -80,84 +80,96 @@ EditControls (Glib::RefPtr<Gtk::TextBuffer> tb_ref_)
 	add_icon_to_factory (ifactory, pixbuf_ref, EditControls::BOLD);
 	m_icon_list.push_back (icon_t (EditControls::BOLD,
 								   manage (new Image (pixbuf_ref)),
-								   "Apply bold tag"));
+								   "Apply bold tag",
+								   "<control>b"));
 	
 	pixbuf_ref = Granule::create_from_file (
 		GRAPPDATDIR "pixmaps/text_italic.png");
 	add_icon_to_factory (ifactory, pixbuf_ref, EditControls::ITALIC);
 	m_icon_list.push_back (icon_t (EditControls::ITALIC,
 								   manage (new Image (pixbuf_ref)),
-								   "Apply italic tag"));
+								   "Apply italic tag",
+								   "<control>i"));
 
 	pixbuf_ref = Granule::create_from_file (
 		GRAPPDATDIR "pixmaps/text_underlined.png");
 	add_icon_to_factory (ifactory, pixbuf_ref, EditControls::UNDERLINE);
 	m_icon_list.push_back (icon_t (EditControls::UNDERLINE,
 								   manage (new Image (pixbuf_ref)),
-								   "Apply underline tag"));
+								   "Apply underline tag",
+								   "<control>u"));
 
 	pixbuf_ref = Granule::create_from_file (
 		GRAPPDATDIR "pixmaps/text_monospaced.png");
 	add_icon_to_factory (ifactory, pixbuf_ref, EditControls::MONOSPACE);
 	m_icon_list.push_back (icon_t (EditControls::MONOSPACE,
 								   manage (new Image (pixbuf_ref)),
-								   "Apply monospace tag"));
+								   "Apply monospace tag",
+								   "<control>t"));
 
 	pixbuf_ref = Granule::create_from_file (
 		GRAPPDATDIR "pixmaps/text_subscript.png");
 	add_icon_to_factory (ifactory, pixbuf_ref, EditControls::SUBSCRIPT);
 	m_icon_list.push_back (icon_t (EditControls::SUBSCRIPT,
 								   manage (new Image (pixbuf_ref)),
-								   "Apply subscript tag"));
+								   "Apply subscript tag",
+								   "<control>l"));
 
 	pixbuf_ref = Granule::create_from_file (
 		GRAPPDATDIR "pixmaps/text_superscript.png");
 	add_icon_to_factory (ifactory, pixbuf_ref, EditControls::SUPERSCRIPT);
 	m_icon_list.push_back (icon_t (EditControls::SUPERSCRIPT,
 								   manage (new Image (pixbuf_ref)),
-								   "Apply superscript tag"));
+								   "Apply superscript tag",
+								   "<control>k"));
 
 	pixbuf_ref = Granule::create_from_file (
 		GRAPPDATDIR "pixmaps/text_big.png");
 	add_icon_to_factory (ifactory, pixbuf_ref, EditControls::BIG);
 	m_icon_list.push_back (icon_t (EditControls::BIG,
 								   manage (new Image (pixbuf_ref)),
-								   "Make selection bigger"));
+								   "Make selection bigger",
+								   "<control>m"));
 
 	pixbuf_ref = Granule::create_from_file (
 		GRAPPDATDIR "pixmaps/text_small.png");
 	add_icon_to_factory (ifactory, pixbuf_ref, EditControls::SMALL);
 	m_icon_list.push_back (icon_t (EditControls::SMALL,
 								   manage (new Image (pixbuf_ref)),
-								   "Make selection smaller"));
+								   "Make selection smaller",
+								   "<control>s"));
 
 	pixbuf_ref = Granule::create_from_file (
 		GRAPPDATDIR "pixmaps/text_remove_tags.png");
 	add_icon_to_factory (ifactory, pixbuf_ref, EditControls::REMOVETAGS);
 	m_icon_list.push_back (icon_t (EditControls::REMOVETAGS,
 								   manage (new Image (pixbuf_ref)),
-								   "Remove all tags"));
+								   "Remove all tags",
+								   "<control>x"));
 
 	pixbuf_ref = Granule::create_from_file (
 		GRAPPDATDIR "pixmaps/text_paste.png");
 	add_icon_to_factory (ifactory, pixbuf_ref, EditControls::PASTE);
 	m_icon_list.push_back (icon_t (EditControls::PASTE,
 								   manage (new Image (pixbuf_ref)),
-								   "Paste from clipboard"));
+								   "Paste from clipboard",
+								   "<control>p"));
 
 	pixbuf_ref = Granule::create_from_file (
 		GRAPPDATDIR "pixmaps/text_normalize.png");
 	add_icon_to_factory (ifactory, pixbuf_ref, EditControls::NORMALIZE);
 	m_icon_list.push_back (icon_t (EditControls::NORMALIZE,
 								   manage (new Image (pixbuf_ref)),
-								   "Remove extra spaces and newlines"));
+								   "Remove extra spaces and newlines",
+								   "<control>n"));
 
 	pixbuf_ref = Granule::create_from_file (
 		GRAPPDATDIR "pixmaps/text_addindex.png");
 	add_icon_to_factory (ifactory, pixbuf_ref, EditControls::ADDINDEX);
 	m_icon_list.push_back (icon_t (EditControls::ADDINDEX,
 								   manage (new Image (pixbuf_ref)),
-								   "Add reference index"));
+								   "Add reference index",
+								   "<control>r"));
 
 	ifactory->add_default ();
 
@@ -183,8 +195,10 @@ EditControls (Glib::RefPtr<Gtk::TextBuffer> tb_ref_)
 											  (*iter).get_stock_id (),
 											  (*iter).get_name (),
 											  (*iter).get_tooltip ()),
-		 bind<string>(mem_fun (*this, &EditControls::on_edit_button_clicked),
-					  (*iter).get_name ()));
+						 Gtk::AccelKey ((*iter).get_accel_key ()),
+						 bind<string>(mem_fun (*this, 
+										 &EditControls::on_edit_button_clicked),
+									  (*iter).get_name ()));
 		iter++;
 	}
 
