@@ -1,10 +1,10 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: PrefWindow.cpp,v 1.16 2008/01/06 16:51:10 vlg Exp $
+// $Id: PrefWindow.cpp,v 1.17 2008/06/02 03:03:54 vlg Exp $
 //------------------------------------------------------------------------------
 //                            PrefWindow.h
 //------------------------------------------------------------------------------
-//  Copyright (c) 2004,2006 by Vladislav Grinchenko
+//  Copyright (c) Vladislav Grinchenko
 //
 //  This program is free software; you can redistribute it and/or 
 //  modify it under the terms of the GNU General Public License   
@@ -52,7 +52,6 @@ PrefWindow (Gtk::Widget& parent_)
 	 */
 	append_page (m_general_pref,    "General"   );
 	append_page (m_appearance_pref, "Appearance");
-//	append_page (m_scheduling_pref, "Scheduling");
 	append_page (m_sound_pref,      "Sound"     );
 	append_page (m_geometry_pref,   "Geometry"  );
 }
@@ -70,15 +69,15 @@ run ()
 	 */
 	m_general_pref.   load_from_config ();
 	m_appearance_pref.load_from_config ();
-//	m_scheduling_pref.load_from_config ();
 	m_sound_pref.     load_from_config ();
 	m_geometry_pref.  load_from_config ();
 	
 	m_ignore_callbacks = false;
 	m_apply_button->set_sensitive (false);
 
-    show ();
-    Gtk::Main::run ();
+	show ();
+    present ();
+    gtk_main ();
 
     return m_changed;
 }
@@ -91,7 +90,38 @@ apply_impl ()
 
 	m_general_pref.   save_to_config ();
 	m_appearance_pref.save_to_config ();
-//	m_scheduling_pref.save_to_config ();
 	m_sound_pref.     save_to_config ();
 	m_geometry_pref.  save_to_config ();
 }
+
+void
+PrefWindow::
+stop ()
+{
+    trace_with_mask("PrefWindow::stop",GUITRACE);
+
+    hide ();
+    Gtk::Main::quit ();
+}
+
+void 
+PrefWindow::
+changed_cb ()
+{
+    trace_with_mask("PrefWindow::changed_cb",GUITRACE);
+
+	m_changed = true;
+	DL ((GRAPP,"m_changed set to true\n"));
+
+	if (!m_ignore_callbacks) {
+		changed ();				// Enable <Apply> button
+	}
+}
+
+void 
+PrefWindow::
+close_impl ()
+{
+    trace_with_mask("PrefWindow::close_impl",GUITRACE);
+    stop ();
+}
