@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: TextAlignment.cpp,v 1.9 2007/12/02 00:31:26 vlg Exp $
+// $Id: TextAlignment.cpp,v 1.10 2008/08/16 03:13:09 vlg Exp $
 //------------------------------------------------------------------------------
 //                            TextAlignment.cpp
 //------------------------------------------------------------------------------
@@ -40,25 +40,30 @@ TextAlignment (PrefWindow& grandpa_)
 	TextAlignmentsUI::value_list_t align_x_list;
 	TextAlignmentsUI::value_list_t align_y_list;
 	TextAlignmentsUI::value_list_t paragraph_list;
+	TextAlignmentsUI::value_list_t img_position_list;
 
-	align_x_list = TextAlignmentsUI::make_align_x_list ();
-	align_y_list = TextAlignmentsUI::make_align_y_list ();
-	paragraph_list = TextAlignmentsUI::make_paragraph_list ();
+	align_x_list      = TextAlignmentsUI::make_align_x_list      ();
+	align_y_list      = TextAlignmentsUI::make_align_y_list      ();
+	paragraph_list    = TextAlignmentsUI::make_paragraph_list    ();
+	img_position_list = TextAlignmentsUI::make_img_position_list ();
 
 	m_front_widget = new TextAlignmentsUI ("<b>Front Text Alignment</b>",
 										   align_x_list,
 										   align_y_list,
-										   paragraph_list);
+										   paragraph_list,
+										   img_position_list);
 
 	m_back_widget = new TextAlignmentsUI ("<b>Back Text Alignment</b>",
 										  align_x_list,
 										  align_y_list,
-										  paragraph_list);
+										  paragraph_list,
+										  img_position_list, false);
 
 	m_example_widget = new TextAlignmentsUI ("<b>Example Text Alignment</b>",
 											 align_x_list,
 											 align_y_list,
-											 paragraph_list);
+											 paragraph_list,
+											 img_position_list);
 	/** Pack all together
 	 */
 	m_vbox->pack_start (*(m_front_widget->frame   ()), Gtk::PACK_SHRINK, 4);
@@ -83,6 +88,9 @@ TextAlignment (PrefWindow& grandpa_)
 	m_front_widget->paragraph_combo ()->signal_changed ().connect (
 		mem_fun (m_pref_window, &PrefWindow::changed_cb));
 
+	m_front_widget->img_position_combo ()->signal_changed ().connect (
+		mem_fun (m_pref_window, &PrefWindow::changed_cb));
+
 	/// Back
 	m_back_widget->x_align_combo ()->signal_changed ().connect (
 		mem_fun (m_pref_window, &PrefWindow::changed_cb));
@@ -115,6 +123,9 @@ TextAlignment (PrefWindow& grandpa_)
 	m_example_widget->paragraph_combo ()->signal_changed ().connect (
 		mem_fun (m_pref_window, &PrefWindow::changed_cb));
 
+	m_example_widget->img_position_combo ()->signal_changed ().connect (
+		mem_fun (m_pref_window, &PrefWindow::changed_cb));
+
 	show_all_children ();
 }
 
