@@ -2,7 +2,7 @@
 //------------------------------------------------------------------------------
 //                              TopMenuBar.cpp
 //------------------------------------------------------------------------------
-// $Id: TopMenuBar.cpp,v 1.33 2008/02/10 20:32:17 vlg Exp $
+// $Id: TopMenuBar.cpp,v 1.34 2008/06/16 02:36:53 vlg Exp $
 //------------------------------------------------------------------------------
 //  Copyright (c) 2004,2006-2007 by Vladislav Grinchenko
 //
@@ -37,6 +37,10 @@ create ()
     trace("TopMenuBar::create");
     using namespace Gtk;
 
+	if (m_created) {
+		return;
+	}
+
     m_menu_bar = manage (new MenuBar ());
     m_open_recent_menu = manage (new Menu ());
     {
@@ -234,6 +238,7 @@ create ()
 		m_exit = &f.back ();
 #endif
 
+		m_created = true;
     }
 }
 
@@ -305,10 +310,16 @@ get_menu_bar ()
 
 void
 TopMenuBar::
-set_sensitivity (WMState state_)
+set_sensitivity (MWState state_)
 {
     trace("TopMenuBar::set_sensitivity");
 
+	if (! is_created ()) {
+		return;
+	}
+
+	m_mw.dump_mw_state ();
+
     switch (state_) 
     {
     case Start:
@@ -365,7 +376,12 @@ set_deck_sensitivity (bool is_deck_selected_, size_t num_open_decks_)
 {
     trace("TopMenuBar::set_deck_sensitivity");
 
-	if (is_deck_selected_) {
+	if (!is_created ()) {
+		return;
+	}
+
+	if (is_deck_selected_) 
+	{
 		m_deck_new      ->set_sensitive (false);
 		m_deck_open     ->set_sensitive (false);
 		m_deck_insert   ->set_sensitive (true );
