@@ -2,7 +2,7 @@
 //------------------------------------------------------------------------------
 //                              TopMenuBar.h
 //------------------------------------------------------------------------------
-// $Id: TopMenuBar.h,v 1.18 2007/12/26 03:32:23 vlg Exp $
+// $Id: TopMenuBar.h,v 1.19 2008/06/16 02:36:53 vlg Exp $
 //------------------------------------------------------------------------------
 //  Copyright (c) 2004,2006-2007 by Vladislav Grinchenko
 //
@@ -43,11 +43,14 @@ public:
     ~TopMenuBar ();
     
     void     create ();
+	bool     is_created () const { return m_created; }
+
     MenuBar* get_menu_bar ();
+
     void     add_history ();
     void     refill_history_list ();
     
-    void set_sensitivity (WMState state_);
+    void set_sensitivity (MWState state_);
     void set_deck_sensitivity (bool is_deck_selected_, size_t num_open_decks_);
 
 #ifdef IS_HILDON
@@ -62,6 +65,7 @@ private:
 	MainWindow&  m_mw;			// These are needed because of c'tor's
 	DeckManager& m_dm;			// circular dependency
 
+	bool         m_created; 
     bool         m_loaded;	// MainWin to let us know when list is loaded
 
     MenuBar*     m_menu_bar;
@@ -111,6 +115,7 @@ TopMenuBar::
 TopMenuBar (MainWindow& mw_, DeckManager& dm_) :
 	m_mw (mw_),
 	m_dm (dm_),
+	m_created (false),
 	m_file_menu (NULL), 
 	m_new (NULL),           
 	m_open (NULL),      	m_open_recent (NULL),   
